C*************************************************************************
C     FORTRAN77 code
C     File: dynamic.for
C     Coded by: Antoni Vidiella-i-Anguera
C
C     A SIMULATION PROGRAM FOR SEASETAR PROCESSES
C     N  = SAMPLE SIZE OF THE SERIES
C     IS = PERIOD OF SEASONALITY
C************************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      INTEGER THMAX,SEED
C
C     *** PARAMETER VARIABLES ***
C
      PARAMETER (LMAX=100,NMAX=5000,THMAX=1,NSIMAX=200)
C
C     *** REAL VECTORS AND MATRICES ***
C      
      DIMENSION Y(NMAX,NSIMAX),THD(THMAX),CO(2)
      DIMENSION XI(LMAX),Y3(NMAX),YKL(1,1),YI(1,LMAX),B11(LMAX,1)
      DIMENSION B12(LMAX,1),B3(2,LMAX),RI(1000),Y4(1,LMAX)
C
C     *** EXTERNAL SUBROUTINES ***
C
      EXTERNAL ZEROMAT,ZEROVEC,GNORM,EQMAT,EQVEC,MULTMAT

C
C     *** SIMULATION DATA ***
c      NSIM=2
      N= 120
      write(6,*) 'nsim?'
      read(5,*) nsim
c
C     *** OPEN RESULTS FILE ***
C
      OPEN (8,FILE='simul.res',STATUS='UNKNOWN')
      OPEN (3,FILE='data.dat',STATUS='UNKNOWN')

C**********************************************************************
C     SIMULATION SEASONAL THERSHOLD AUTOREGRESSIVE PROCESS 
C
C**********************************************************************
      ST1=0.0
      ST2=0.0
      CALL ZEROMAT(B3,2,2,LMAX)
      ID=0
      CALL ZEROVEC(CO,2,2)
      READ(3,*) NPAM1
      READ(3,*) CO(1)
      DO 30 I1=1,NPAM1
 30      READ(3,*) B3(1,I1)
      READ(3,*) NPAM2
      READ(3,*) CO(2)
      DO 40 I1=1,NPAM2
 40      READ(3,*) B3(2,I1)
      READ(3,*) THD(1)
      READ(3,*) ST1
      READ(3,*) ST2
      READ(3,*) ID
      SEED=1234567890
         DO 12 IT0=1,NPAM1
 12         B11(IT0,1)=B3(1,IT0)
         DO 14 IT0=1,NPAM2
 14         B12(IT0,1)=B3(2,IT0)
         READ(3,*) NPAM
         DO 16 I1=0,NPAM-1
            I2=NPAM-I1
 16         READ(3,*) YI(1,I2)
      CALL EQMAT(YI,Y4,1,1,NPAM)
C     *** SIMULATION START ***
      DO 150 I=1,NSIM
C
C     *** SEASETAR(IP1,IP2,S) MODEL GENERATION (non-multiplicative)*** 
C
         CALL EQMAT(Y4,YI,1,1,NPAM)
         CALL ZEROVEC(Y3,NMAX,N)
         CALL ZEROVEC(RI,1000,1000)
         CALL GNORM(RI,1000,SEED)
            DO 15 IT3 = 1,N+200
               IF(YI(1,ID).LE.THD(1)) THEN                  
                  CALL MULTMAT(YI,B11,YKL,1,NMAX,1,NPAM1,1)
c                  write(8,*) CO(1),YKL(1,1),RI(250+IT3)*ST1,'inf'
                  Y3(IT3)=CO(1)+YKL(1,1)+RI(250+IT3)*ST1
               ELSE IF(YI(1,ID).GT.THD(1)) THEN
                  CALL MULTMAT(YI,B12,YKL,1,NMAX,1,NPAM2,1)
c                  write(8,*) CO(2),YKL(1,1),RI(250+IT3)*ST2,'sup'
                  Y3(IT3)=CO(2)+YKL(1,1)+RI(250+IT3)*ST2
               END IF
               DO 17 IT0=0,NPAM-2
                  IK=NPAM-IT0
 17               YI(1,IK)=YI(1,IK-1)
               YI(1,1)=Y3(IT3)
 15      CONTINUE
         DO 18 I1=1,N
 18         Y(I1,I)=Y3(I1+200)
c         write(6,*) i
 150  CONTINUE
      DO 20 I1=1,N
 20      WRITE(8,*) (Y(I1,I2),I2=1,NSIM)
c 900  FORMAT (F10.5)
      CLOSE (8)
      CLOSE (4)
      END

      SUBROUTINE ZEROMAT(AMAT,ID1MAX,L1,L2)
C**************************************************************
C     PURPOSE: CREATE A MATRIX OF ZEROS
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C     OUTPUT: AMAT, ZEROED MATRIX DIMENSION L1,L2
C**************************************************************      
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*)
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    AMAT(I,J)=0.0
  99     CONTINUE
  98  CONTINUE
      RETURN
      END

      SUBROUTINE ZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PURPOSE: CREATE A VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VEC(ID1MAX)
      DO 98 I=1,L1     
         VEC(I)=0.0
  98  CONTINUE
      RETURN
      END   

      SUBROUTINE EQVEC(VECA,VECB,ID1MAX,L1)
C******************************************************************
C     PURPOSE: EQUALIZATION OF VECTORS
C     INPUT: VECA, VECTOR DIMENSION L1
C     OUTPUT: VECB, VECTOR EQUAL VECA DIMENSON L1
C******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VECA(ID1MAX),VECB(ID1MAX)
C
      DO 99 I=1,L1
         VECB(I)=VECA(I)
   99 CONTINUE
      RETURN
      END

      SUBROUTINE MULTMAT(AMAT,BMAT,CMAT,ID1MAX,ID2MAX,L1,L2,L3)
C**********************************************************************
C     PORPOSE: MATRIX MULTIPLICATION
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C            BMAT, MATRIX DIMENSION L2,L3
C     OUPUT: CMAT, MATRIX DIMENSION L1,L3
C**********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID2MAX,*),CMAT(ID1MAX,*)
      EXTERNAL ZEROMAT
      CALL ZEROMAT(CMAT,ID1MAX,L1,L3)
      DO 99 J=1,L3
	 DO 98 K=1,L2
	    IF (BMAT(K,J).EQ.0.0) GOTO 98
	    DO 97 I=1,L1
	       CMAT(I,J)=CMAT(I,J)+AMAT(I,K)*BMAT(K,J)
   97       CONTINUE
   98    CONTINUE
   99 CONTINUE
      RETURN
      END

       subroutine gnorm(r,nr,rs)
C       IMPLICIT REAL (A-H,O-Z)
       integer nr,rs
       real*8 r(nr)
       do  1 i=1,nr
99        continue         
          v=randu(rs)*2.-1.
          u=randu(rs)*2.-1.
          w=v*v+u*u
          if(w.gt.1)goto 99
          c=sqrt(-2.*alog(w)/w)
          R(i)=c*v
          r(i+1)=c*u
1      continue
       return
       end                      
	   	   
C**************************************************************************  
C  
      REAL FUNCTION RANDU(IX)  
C  
C UNIFORM RANDOM NUMBER GENERATOR.  
C REF: SCHRAGE, L. 1979 'A MORE PORTABLE FORTRAN RANDOM NUMBER GENERATOR'  
C      A. C. M. TRANSACTIONS ON MATHEMATICAL SOFTWARE V5 132-138.  
C  
      INTEGER A,P,IX,B15,B16,XHI,XALO,LEFTLO,FHI,K  
      DATA A/16807/,B15/32768/,B16/65536/,P/2147483647/  
      XHI=IX/B16  
      XALO=(IX-XHI*B16)*A  
      LEFTLO=XALO/B16  
      FHI=XHI*A+LEFTLO  
      K=FHI/B15  
      IX=(((XALO-LEFTLO*B16)-P)+(FHI-K*B15)*B16)+K  
      IF (IX.LT.0) IX=IX+P  
      RANDU=FLOAT(IX)*4.656612875E-10  
      RETURN  
      END  
C

      SUBROUTINE EQMAT(AMAT,BMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: EQUALIZATION OF MATRICES 
C     INPUT: AMAT, MATRIX DIMENSION L1 L2
C     OUPUT: BMAT, MATRIX EQUAL AMAT DIMENSION L1 L2
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*)
C
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    BMAT(I,J)=AMAT(I,J)
   99    CONTINUE
   98 CONTINUE
      RETURN
      END
