C*************************************************************************
C       FORTRAN77 code
C       Coded by: Antoni Vidiella-i-Anguera
C       File: Seasid.for
C
C       PROGRAM FOR IDENTIFICATION OF SEASETAR MODELS
C       SEPTEMBER 2000 modified in november 01 
C       N   = SAMPLE SIZE OF THE SERIES
C       ID1 = DEGREE OF DIFFERENCING ( APPLIED OVER TRANSFORMED DATA!)
C       ID2 = DEGREE OF SEASONAL DIFFERENCING
C       IDX = MAX DELAY VALUE
C       IP2 = ORDER OF SEASONAL MAX AR OPERATOR
C       B0  = INITIAL ESTIMATE VECTOR
C       B1  = FINAL ESTIMATE VECTOR
C       A1  = VECTOR OF STD. ERRORS
C       IS  = PERIOD OF SEASONALITY
C       S2  = ESTIMATE OF RESIDUAL VARIANCE
C       I0  = MAXIMUM NUMBER OF ITERATIONS
C       ISEL = SELECTION CRITERIA (0 AICC, 1 AICU)
C       ITR=  0, USE RAQ DATA
C           1, USE LOGARITHM OF DATA
C           2, USE LOG 1ST DIFFERENCES OF DATA
C           3, USE RAW (NO LOG) 1ST DIFFERENCES OF DATA
C           4, USE LOG-LINEAR DETRENDED DATA
C           5, USE RAW LINEAR DETRENDED DATA
C
C       METHOD OF ESTIMATION: LEAST SQUARES
C************************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER THMAX
C
C     *** CHARACTER VARIABLES ***
C
      CHARACTER*12 FILEI,FILEO
      CHARACTER*50 DESC
C
C     *** PARAMETER VARIABLES ***
C
      PARAMETER (LMAX=50,NMAX=500,THMAX=1)


C
C     *** REAL VECTORS AND MATRICES ***
C      
      DIMENSION Y(NMAX),C(LMAX),AC2(LMAX),ZTHD(5000,7)
      DIMENSION AX(NMAX,LMAX,THMAX+1),THD(THMAX),KKARAY(THMAX+1)
      DIMENSION NOBS(THMAX+1),TIME(NMAX),SAX(NMAX,LMAX)
      DIMENSION POST(NMAX),SAICC(NMAX),IPAZ(LMAX),acx(nmax)
      DIMENSION Y1(NMAX),SAICU(NMAX),SAG(NMAX,LMAX),I2PAZ(LMAX)      
C
C     *** EXTERNAL SUBROUTINES ***
C
      EXTERNAL ZEROMAT,ZEROVEC
      EXTERNAL sort,IZEROVEC,ZEROMAT3
      EXTERNAL HUSHLD,EQVEC,accum
      EXTERNAL ARMFIT,VARCOV,SINV,PCAND,QCKSRT
C
C     READ INPUT FILE
C
      OPEN (7,FILE='inf1.dat',STATUS='UNKNOWN')
      READ (7,9020) DESC
      READ (7,*) N
      READ (7,*) ID1
      READ (7,*) ID2
      READ (7,*) IIP2
      READ (7,*) IS
      READ (7,*) I0
      READ (7,*) ITR
      READ (7,*) NTHD
      READ (7,*) IDX
      READ (7,*) ISEL
      READ (7,9021) FILEO
      READ (7,9021) FILEI
      READ (7,*) PL
      READ (7,*) PU
c      IF (IP1.LT.0) THEN
c         WRITE (6,*) 'Error: P1 of AR(p) is negative!!'
c         STOP
c      END IF
c      IF (IP2.LT.0) THEN
c         WRITE(6,*) 'Error: P2 of SAR(p) is negative!!'
c        STOP
c      END IF
C
C     READ IN DATA FROM FILE "FILEI"
C
      OPEN(9,FILE=FILEI,STATUS='UNKNOWN')
      DO 12,IT=1,N
           READ(9,*) Y(IT)
 12    CONTINUE 
C
C     *** OPEN OUTPUT FILE ***
C
        OPEN (8,FILE=FILEO,STATUS='UNKNOWN')
C
C     *** LIMITING NUMBER OF ITERATIONS
C
        IF (I0.GT.70) I0=70


C
C     *** DATA TRANSFORMATIONS ***
C
      IF ((ITR.EQ.1).OR.(ITR.EQ.2).OR.(ITR.EQ.4)) THEN
        DO 11 M=1,N
 11        Y(M)=LOG(Y(M))
      END IF
      IF ((ITR.EQ.2).OR.(ITR.EQ.3)) THEN
        N = N-1
        DO 6 M=1,N
 6          Y(M)=Y(M+1)-Y(M)
      END IF
      IF ((ITR.EQ.4).OR.(ITR.EQ.5)) THEN
        DO 7 M=1,N
 7         TIME(M)=REAL(M)
      END IF
C
C     *** DIFFERENCING & SEASONAL DIFFERENCING ***
C
        IF (ID1.NE.0) THEN
         DO 1 I3=1, ID1
              N = N-1
            DO 1 I4=1, N
 1             Y(I4) = Y(I4+1) - Y(I4)
      END IF
      IF (ID2.NE.0) THEN
           DO 3 I3=1, ID2
              N = N - IS
              DO 4 I4=1, N
                   Y(I4) = Y(I4+IS) - Y(I4)
  4         CONTINUE
  3      CONTINUE
      END IF
      N1=N
      IS1=IS      
      IIP1M=IS-1


C**********************************************************************
C     SEARCH PROCEDURE OF THE BEST LINEAR NON MULTIPLICATIVE SEASONAL
C     AUTOREGRESIVE PROCES 
C     
C**********************************************************************
      IER=0
      IES=0
      THD(1)=0.0
      KKARAY(1)=(IS-1)+IIP2*IS
      CALL ZEROMAT3(AX,NMAX,LMAX,N,IIP1M,THMAX+1)
      CALL ZEROVEC(C,LMAX,KKARAY(1))
      CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,0,1,NOBS,IER,IES,is)
      N=NOBS(1)
      ITF=KKARAY(1)
      ICP=1
      IC=0
      DO 5126 IP2=0,IIP2
         DO 5128 IP1=0,IS-1
              IC=IC+1
              KR=IP1+IP2*IS
              KRL=IP1+1
              K1=KR+2
              K2=KR+1
                IL=IP1+IP2*KRL
                CALL ZEROMAT(SAX,NMAX,N,K1)
                DO 5129 I1=1,N
                   DO 5131 I8=1,IP2+1
                      DO 5131 I2=1,KRL
                   SAG(I1,(I8-1)*KRL+I2)=AX(I1,IS*(I8-1)+I2,1)
 5131          CONTINUE        
                   SAG(I1,(I8-1)*KRL+1)=AX(I1,ITF+2,1)
 5129       CONTINUE
                CALL HUSHLD(SAG,N,IL+2,NMAX)
                CALL ARMFIT(SAG,NMAX,AC2,IL+1,N,VA,IMIN,AICC,AICU)
                SAICC(IC)=AICC
                SAICU(IC)=AICU
                IF (ISEL.EQ.0) SAICU(IC)=0.0
                IF (ISEL.EQ.1) SAICC(IC)=0.0
                ZTHD(IC,1)=REAL(IP1)
                ZTHD(IC,2)=REAL(IP2)
                ZTHD(IC,3)=DMIN1(SAICC(IC),SAICU(IC))
               write(8,*) (zthd(ic,ik),ik=1,3)
 5128     CONTINUE
 5126  CONTINUE
       IOP=1
       ZTHDM=ZTHD(1,3)
       DO 5106 I=2,IC
            IF (ZTHD(I,3).LT.ZTHDM) THEN 
               ZTHDM=ZTHD(I,3)
               IOP=I
            END IF
 5106  CONTINUE
        write(8,*) desc
        write(8,*) (zthd(iop,ik),ik=1,3)
C**********************************************************************
C    SEARCH PROCEDURE OF THE BEST NON MULTIPLICATIVE SEASONAL
C    THRESHOLD AUTOREGRESIVE PROCES 
C**********************************************************************
       NC=1
       PAIC=0.0
       PSBIC=0.0
       PAIC2=0.0
       PSBIC2=0.0
       IC=0
       N=N1
         IND=1
       CALL ZEROVEC(SAICC,NMAX,NMAX)
       CALL ZEROVEC(SAICU,NMAX,NMAX) 


C   *** GENERATE VECTOR OF POSSIBLE THRESHOLDS ***
       CALL EQVEC(Y,Y1,NMAX,N)
       irrev=1
       call accum(y1,N,IS,acx,irrev)
       CALL PCAND(acx,N-is,POST,PL,PU,IPA)
c       do 93 i1=1,ipa
c 93       write(8,*) post(i1)
c       stop
       CALL IZEROVEC(KKARAY,THMAX+1,NTHD+1)
C   *** BEGIN LOOP ESTIMATION OF MODELS ***
       DO 91 I=1,2
 91       KKARAY(I)=(IS-1)+IS*IIP2
       DO 48 ID=1,IDX
       DO 96 IP2A=0,IIP2
            DO 98 IP2B=0,IIP2
               DO 102 J1=0,IS-1
                    DO 104 J2=0,IS-1
                   IC=IC+1
                   write(6,*) ic,j1,j2,ip2a,ip2b,id
                   i2paz(1)=ip2a
                   i2paz(2)=ip2b
                   ipaz(1)=j1
                   ipaz(2) =j2        
                   DO 125 I3=1,IPA
                      THD(1)=POST(I3)
                      N=N1
                      IER=0
                      IES=0
                      CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
                      CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,NTHD
     1                ,ID,NOBS,IER,IES,is)
                      IF (IER.EQ.1) GOTO 125
C    *** BEGIN LOOP PIECES TO BE ESTIMATED ***           
                      DO 145 ICP=1,NTHD+1
                     KR=KKARAY(ICP)
                           IP2=I2PAZ(ICP)
                           IP1=IPAZ(ICP)
                           KRL=IP1+1
                           N=NOBS(ICP)
                           K1=KR+2
                           K2=KR+1
                           IL=IP1+IP2*KRL
                           CALL ZEROMAT(SAX,NMAX,N,K1)
                           DO 129 I1=1,N
                              DO 131 I8=1,IP2+1
                                DO 131 I2=1,KRL
                                  SAG(I1,(I8-1)*KRL+I2)=
     1                           AX(I1,IS*(I8-1)+I2,ICP)
 131                  CONTINUE        
                      SAG(I1,(I8-1)*KRL+1)=AX(I1,K1,ICP)
 129               CONTINUE                         
                         CALL HUSHLD(SAG,N,IL+2,NMAX)
                         CALL ARMFIT(SAG,NMAX,AC2,IL+1,N,VA,
     1                   IMIN,AICC,AICU)
                         SAICC(I3)=SAICC(I3)+AICC
                         SAICU(I3)=SAICU(I3)+AICU
 145                  CONTINUE
 125               CONTINUE 
C     *** SELECTION PROCEDURE ***
                   IF (ISEL.EQ.0) THEN
                  CALL ZEROVEC(SAICU,NMAX,NMAX)                  
                      D10=SAICC(1)
                   ELSE 
                  CALL ZEROVEC(SAICC,NMAX,NMAX)
                      D10=SAICU(1)
                   END IF
                   IOP=1
                   DO 185 I=2,IPA
                      IF (SAICC(I).LT.D10.AND.ISEL.EQ.0) THEN
                           D10=SAICC(I)
                           IOP=I
                      END IF
                      IF (SAICU(I).LT.D10.AND.ISEL.EQ.1) THEN
                           D10=SAICU(I)
                           IOP=I
                      END IF
 185              CONTINUE
                   ZTHD(IC,1)=REAL(J1)
                   ZTHD(IC,2)=REAL(J2)
                   ZTHD(IC,3)=POST(IOP)
                   ZTHD(IC,4)=DMIN1(SAICC(IOP),SAICU(IOP))
                   ZTHD(IC,5)=REAL(IP2A)
                   ZTHD(IC,6)=REAL(IP2B)
                   ZTHD(IC,7)=REAL(ID)
                  write(6,*) zthd(ic,4)
                   D10=0.0
                   CALL ZEROVEC(SAICC,NMAX,NMAX)
                   CALL ZEROVEC(SAICU,NMAX,NMAX)
 104            CONTINUE
 102         CONTINUE
 98       CONTINUE
 96    CONTINUE
 48    CONTINUE
       IOP=1
       ZTHDM=ZTHD(1,4)
       DO 106 I=2,IC
          IF (ZTHD(I,4).LT.ZTHDM) THEN 
             ZTHDM=ZTHD(I,4)
             IOP=I
          END IF
 106   CONTINUE
      write(8,*) ' '
      do 300 i=1,2
         write(8,9031) i
 300     write(8,*) int(zthd(iop,i))
      do 310 i=5,6
         write(8,9030) i
 310     write(8,*) int(zthd(iop,i))
      write(8,9032) zthd(iop,3)
      write(8,9033) int(zthd(iop,7))
       write(8,*) (zthd(iop,ij),ij=1,7)
 9020 FORMAT (A50)
 9030 format('Seasonal order regime : ',i5)
 9031 format('Non-seasonal order regime :',i5)
 9032 format('Threshold :',f12.7)
 9033 format('Optimal lag :',i5)
 9021 FORMAT (A12)
 9999 CLOSE(7)
        CLOSE(8)
        CLOSE(9)
      END



      SUBROUTINE ZEROMAT(AMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: CREATE A MATRIX OF ZEROS
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C     OUTPUT: AMAT, ZEROED MATRIX DIMENSION L1,L2
C**************************************************************
      
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*)
      DO 98 J=1,L2
         DO 99 I=1,L1
            AMAT(I,J)=0.0
  99     CONTINUE
  98  CONTINUE
      RETURN
      END


      SUBROUTINE ZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PORPOSE: CREATE A VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
        DIMENSION VEC(ID1MAX)


      DO 98 I=1,L1     
                VEC(I)=0.0
  98  CONTINUE
      RETURN
        END
   


      SUBROUTINE EQVEC(VECA,VECB,ID1MAX,L1)
C******************************************************************
C     PORPOSE: EQUALIZATION OF VECTORS
C     INPUT: VECA, VECTOR DIMENSION L1
C     OUTPUT: VECB, VECTOR EQUAL VECA DIMENSON L1
C******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VECA(ID1MAX),VECB(ID1MAX)
C
      DO 99 I=1,L1
            VECB(I)=VECA(I)
   99 CONTINUE
      RETURN
      END



      SUBROUTINE SORT(Y,N,AX,IAX1,IAX2,KKARAY,THD,NTHD,ID,NOBS,IER
     1 ,IES,is)
C***********************************************************************
C     PORPOSE: SORT THE DATA FOR WHICH AN APPROPIATE SAR-MODEL APPLIES
C     INPUT: Y, DATA
C            N, ITS DIMENSION
C            THD, ARRAY CONTAINING THE THRESHOLDS
C            NTHD+1, NUMBER OF PIECES
C            ID, DELAY PARAMETER
C     OUTPUT:AX, ARRAY CONTAINING THE SORTED DATA
C            IER, ERROR CODE 0 OK
C                            1 SOME PIECES DO NOT HAVE ENOUGH DATA TO
C                                    ESTIMATE THE AR-MODEL
C            IES, START POSITION OF SORTING
C
C            NOBS(I), NUMBER OF DATA IN THE I-TH PIECE
C ***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION NOBS(*),Y(*),THD(*)
      DIMENSION AX(IAX1,IAX2,*),KKARAY(*)
C     *** NUMBER OF PIECES ***
      JEND=NTHD+1
C     *** CLEANING NOBS  ***
      DO 10 I=1,JEND
 10      NOBS(I)=0


      
C     ***  CHOSE APPROPIATE STARTING POSITION  ***
      IES=is+ID-1
      DO 20 I1=1,JEND
         KK=KKARAY(I1)
 20      IES=MAX0(IES,KK)
      IES=IES+1
C     *** SORT X INTO AX'IS ***
      DO 30 I=IES,N
         ICP=1
C     *** ASSIGN TO FIRST PIECE FOR NTHD=0 ***
         IF (NTHD.EQ.0) GOTO 50
C     *** FIRST ASSIGN TO LAST PIECE ***
         ICP=NTHD+1      
C     *** SEE chk BELONGS TO WHICH PIECE
         DO 40 K=1,NTHD
            aaa=0.0
            do 32 j=id,is+id-1
 32            aaa=aaa+y(i-j)          
            IF (aaa.GT.THD(K)) GOTO 40
            ICP = K
C     *** AH, IT'IS FOUND ***
            GOTO 50
  40     CONTINUE
  50     CONTINUE
C     *** INDICATE THE CURRENT POSITION X(I) OCCUPIES IN THE ICP PIECE
         NOBS(ICP)=NOBS(ICP)+1
         NOB=NOBS(ICP)
         KEND=KKARAY(ICP)+1
C     *** ASSIGN AX(NOB,I)
         DO 60 K=1,KEND
           AX(NOB,K,ICP)=Y(I-K+1)
  60     CONTINUE
C     *** REASSIGN AX TO AVOID CASE KK=0
         AX(NOB,1,ICP)=1.0
         KEND=KEND+1
         AX(NOB,KEND,ICP)=Y(I)
  30  CONTINUE
C     *** CHECK IF ENOUGH DATA TO ESTIMATE THE MODEL ***
C     *** it is required max # of parameters + 3
      IER=0
      DO 70 J=1,JEND
         NN=NOBS(J)
         IF (NN.GE.KEND+3) GOTO 70
         IER=1
C     *** The JTH REGIME LACKS DATA, SO RETURN WITH IER=1 ***
         RETURN
  70  CONTINUE
      RETURN
      END
C     
      SUBROUTINE ZEROMAT3(AMAT,ID1MAX,ID2MAX,L1,L2,L3)
C************************************************************
C
C     PORPOSE: CREATE A 3D MATRIX OF ZEROS   
C     INPUT: AMAT, MATRIX TO BE ZEROED
C            L1,L2,L3, DIMENSIONS
C     OUTPUT: AMAT, ZEROED MATRIX
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,ID2MAX,*)


      DO 97 M=1,L3
         DO 98 J=1,L2
            DO 99 I=1,L1
               AMAT(I,J,M)=0.0
  99        CONTINUE
  98     CONTINUE
  97   CONTINUE
C
      RETURN
      END


      SUBROUTINE IZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PORPOSE: CREATE A INTEGER VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      INTEGER ID1MAX,L1,VEC
        DIMENSION VEC(*)


      DO 98 I=1,L1     
                VEC(I)=0
  98  CONTINUE
      RETURN
        END



      SUBROUTINE HUSHLD(AH1,N,K,IA11)
C===================================================================
C Purpose: HOUSHOLDER TRANSFORMATION
C Input : AH1 = array to be transformed;
C         N = number of columns;
C         K = number of rows;
C         IA11 = number of columns of AH1 in the calling program.
C Output: AH1 = transformed array.
C==================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*),Z(1000)
      TOL=1.0E-30
      NK=MIN0(N,K)
      DO 100 II=1,NK
      H=0.0
      DO 10 I=II,N
      Z(I) = AH1(I,II)
   10 H=H+Z(I)*Z(I)
      IF(H .GT. TOL) GOTO 40
      G=0.0
      GOTO 100
   40 G=SQRT(H)
      F=AH1(II,II)
      IF(F.GE.0.0) G=-G
      Z(II)=F-G
      H=H-F*G
      IF(II .EQ. K) GOTO 100
      II1=II+1
      DO 90 J=II1,NK
      S=0.0
      DO 20 I=II,N
   20 S=S+Z(I)*AH1(I,J)
      S=S/H
      DO 30 I=II,N
   30 AH1(I,J)=AH1(I,J)-Z(I)*S
   90 CONTINUE
  100 AH1(II,II)=G
      RETURN
      END


      SUBROUTINE ARMFIT(AH1,IA11,AC1,K,N,VA,IMIN,AC3,AC4)
C*******************************************************************
C Purpose: given the lags for different pieces, compute the AIC
C          for the AR-model in the Ith piece.
C Input: AH1 = housholder transformed array (so it is upper triangular);
C        IA11 = number of rows of AH1 in the calling program;
C        K = number of columns;
C        N = number of rows (number of data);
C        ISEL = indicator for information criterion.
C Output: AR(IMIN-1) = the chosen AR-model;
C         IMIN = the number parameters of the AR-model;
C         AICM = its corresponding information criterion;
C         VA = mean sum of squared errors;
C         AC1 = contains the coefficients of the AR-model.
C Here, AC2 and AC3 are AICc and AICu for each level
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*)
      DIMENSION AC1(*)
      K1=K+1
      K1=MIN0(N,K1)
      OSD=0.0E00
      FN=N
      OSD=OSD+AH1(K1,K1)*AH1(K1,K1)
      AC2=OSD/FN
      AC3=FN*LOG(AC2)+((2*FN*(K+2))/(FN-K-3))
      AC4=AC3+FN*LOG(REAL(FN/(FN-K-2)))
      VA=AC2
      IMIN=K1-1
      IF(IMIN .EQ. 1) GOTO 200
      DO 100 M=IMIN,IMIN
         AC1(M)=AH1(M,K1)/AH1(M,M)
         MM1=M-1
         DO 110 II=1,MM1
            I=M-II
            SUM=DBLE(AH1(I,K1))
            I1=I+1
            DO 120 J=I1,M
  120       SUM=SUM-DBLE(AC1(J))*DBLE(AH1(I,J))
  110    AC1(I)=SUM/AH1(I,I)
  100 CONTINUE
      RETURN
  200 AC1(1)=AH1(1,K1)/AH1(1,1)
      RETURN
      END


      SUBROUTINE VARCOV(SAX,IAX,IMIN,VA,SDM)
C===================================================================
C===================================================================
C PURPOSE: TO PROVIDE THE VARIANCE COVARIANCE MATRIX FOR THE
C          ESTIMATES.
C INPUT: SAX=UPPER TRIANGULAR FORM OF THE DESIGN MATRIX
C        IAX=NO. OF COLUMNS OF SAX IN THE CALLING PROGRAM
C        IMIN: SAX IS OF DIMENSION IMIN BY IMIN
C        VA=SUM OF SQUARED ERRORS
C ACTION: THE SD ESTIMATES AND THE VARIANCE COVARIANCE MATRIX IS PRINTED
C SUBROUTINES CALLED: SINV
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SAX(IAX,*),SDM(*)
      DIMENSION TA(1000)
      EXTERNAL SINV
      IC=0
C     TO COMPRESS THE UPPER TRIANGULAR MATRIX SAX INTO ONE
C     DIMENSIONAL VECTOR TA
      DO 2 I2=1,IMIN
      DO 2 I1=1,I2
      IC=IC+1
    2 TA(IC)=SAX(I1,I2)
C     CALCULATE VA*INVERSE OF THE PRODUCT OF SAX TRANSPOSE AND SAX
      CALL SINV(TA,IMIN,VA)   
C     RETIREVE THE DIAGONAL AND TAKE THE SQUARE ROOT; THEY ARE
C     ESTIMATES OF STANDARD ERROR
      I=1
      IP=1
 11   SDM(I)=SQRT(TA(IP))
      I=I+1
      IP=I+IP
      IF(I .GT. IMIN) GOTO 10
      GOTO 11
  10  RETURN
      END


      SUBROUTINE SINV(A,N,VA)
C===================================================================
C===================================================================
C PURPOSE: TO PROVIDE THE PRODUCT OF THE INVERSE A'*A AND VA
C INPUT: A THE UPPER TRIANGULAR MATRIX STACKED UP ROW-WISE
C        N=DIMENSION OF THE TRIANGULAR MATRIX,SO N*(N-1)=LENGTH OF A
C        VA=MEAN SUM OF SQUARE ERROR
C OUTPUT: A=THE DESIRED PRODUCT AS DESCRIBED IN THE PURPOSE
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1000)
      IPIV=N*(N+1)/2
      IND=IPIV
      DO 6 I=1,N
      DIN=1.D0/DBLE(A(IPIV))
      A(IPIV)=DIN
      MIN=N
      KEND=I-1
      LANF=N-KEND
      IF(KEND) 5,5,2
    2 J=IND
      DO 4 K=1,KEND
      WORK=0.D0
      MIN=MIN-1
      LHOR=IPIV
      LVER=J
      DO 3 L=LANF,MIN
      LVER=LVER+1
      LHOR=LHOR+L
    3 WORK=WORK+DBLE(A(LVER)*A(LHOR))
      A(J)=-WORK*DIN
    4 J=J-MIN
    5 IPIV=IPIV-MIN
    6 IND=IND-1
      DO 8 I=1,N
      IPIV=IPIV+I
      J=IPIV
      DO 8 K=I,N
      WORK=0.D0
      LHOR=J
      DO 7 L=K,N
      LVER=LHOR+K-I
      WORK=WORK+DBLE(A(LHOR)*A(LVER))
    7 LHOR=LHOR+L
      A(J)=WORK*DBLE(VA)
    8 J=J+K
      RETURN
      END
C#####################################################
C Quicksort routine from numerical recipes
C#####################################################
      SUBROUTINE QCKSRT(N,ARR)
      IMPLICIT REAL*8 (A-H,O-Z)
      PARAMETER (M=7,NSTACK=50,FM=7875.,FA=211.,FC=1663.
     *    ,FMI=1.2698413E-4,maxdat=1000)
      DIMENSION ISTACK(NSTACK)
      DIMENSION ARR(maxdat)
C     DOUBLE PRECISION A
      JSTACK=0
      L=1
      IR=N
      FX=0.
10    IF(IR-L.LT.M)THEN
        DO 13 J=L+1,IR
          A=ARR(J)
          DO 11 I=J-1,1,-1
            IF(ARR(I).LE.A)GO TO 12
            ARR(I+1)=ARR(I)
11        CONTINUE
          I=0
12        ARR(I+1)=A
13      CONTINUE
        IF(JSTACK.EQ.0)RETURN
        IR=ISTACK(JSTACK)
        L=ISTACK(JSTACK-1)
        JSTACK=JSTACK-2
      ELSE
        I=L
        J=IR
        FX=MOD(FX*FA+FC,FM)
        IQ=L+(IR-L+1)*(FX*FMI)
        A=ARR(IQ)
        ARR(IQ)=ARR(L)
20      CONTINUE
21        IF(J.GT.0)THEN
            IF(A.LT.ARR(J))THEN
              J=J-1
              GO TO 21
            ENDIF
          ENDIF
          IF(J.LE.I)THEN
            ARR(I)=A
            GO TO 30
          ENDIF
          ARR(I)=ARR(J)
          I=I+1
22        IF(I.LE.N)THEN
            IF(A.GT.ARR(I))THEN
              I=I+1
              GO TO 22
            ENDIF
          ENDIF
          IF(J.LE.I)THEN
            ARR(J)=A
            I=J
            GO TO 30
          ENDIF
          ARR(J)=ARR(I)
          J=J-1
        GO TO 20
30      JSTACK=JSTACK+2
        IF(JSTACK.GT.NSTACK)PAUSE 'NSTACK must be made larger.'
        IF(IR-I.GE.I-L)THEN
          ISTACK(JSTACK)=IR
          ISTACK(JSTACK-1)=I+1
          IR=I-1
        ELSE
          ISTACK(JSTACK)=I-1
          ISTACK(JSTACK-1)=L
          L=I+1
        ENDIF
      ENDIF
      GO TO 10
      END


      SUBROUTINE PCAND(X,N,CTHD,IPL,IPU,IPR)
C**************************************************************
C DEFINES CANDIDATE THRESHOLD VALUES FOR INFLATION RATES
C THRESHOLD VALUES ARE SEARCHED AMONG THE INTERPERCENTILE RANGE
C @ EQUIWIDTH INTERVALS ( # INTERVALS = # VALUES BETWEEN PERC)
C X = MATRIX OF DATA
C N = # DATA
C CTHD = ARRAY OF CANDIDATE THRESHOLD VALUES
C IPL = USER-SUPPLIED LOWER PERCENTILE
C IPU = USER-SUPPLIED UPPER PERCENTILE
C EXTERNAL SUBROUTINE = QCKSRT
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 IPL,IPU
      DIMENSION X(*),CTHD(*),SARR(1000)
      EXTERNAL QCKSRT
      DO 12 I=1,N
 12      SARR(I)=X(I)
      CALL QCKSRT(N,SARR)
      IP1=NINT(IPL*N)
      AMIN=SARR(IP1)
      IP2=NINT(IPU*N)
      AMAX=SARR(IP2)
      IPR=IP2-IP1
      AI=AMAX-AMIN
      AINC=AI/REAL(IPR)
      CTHD(1)=AMIN
      DO 13 J=2,IPR
 13      CTHD(J)=CTHD(J-1)+AINC
      RETURN
      END


      SUBROUTINE accum(X,N,IS,acx,irrev)
C**************************************************************
C DEFINES CANDIDATE THRESHOLD VALUES FOR accumulated
c INFLATION RATES
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*),acx(*)
      if (irrev.eq.0) then
         do 2 i=1,n-is+1
            do 2 j=1,is      
 2             acx(i)=acx(i)+x(i+j-1)
      else
         do 4 i=1,n-is+1
            do 4 j=1,is      
 4             acx(i)=acx(i)+x(n-i-j+2)
      endif
      return
      end
