C*************************************************************************
C     FORTRAN77 code
C     File: gappow.for
C     Coded by: Antoni Vidiella-i-Anguera
C
C     SIMULATION PROGRAM SEASONAL MODELS GAP & LM TEST POWER
C     METHOD OF ESTIMATION: SCORING & LEAST SQUARES
C************************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER THMAX,SEED,I0

C
C     *** PARAMETER VARIABLES ***
C
      PARAMETER (LMAX=10,NMAX=350,THMAX=1)
C
C     *** REAL VECTORS AND MATRICES ***
C
      DIMENSION B0(LMAX),B1(LMAX),Y(NMAX),B11(LMAX,1),ELP(LMAX,LMAX)
      DIMENSION A2(LMAX),A(LMAX,LMAX),B(LMAX,LMAX),Q(LMAX,LMAX)
      DIMENSION C2(LMAX),A0(LMAX),B9(LMAX,1),AC1(LMAX),Y1(NMAX)
      DIMENSION B2(LMAX,LMAX),SDM(LMAX),SX(NMAX,LMAX),AC3(LMAX)
      DIMENSION AX(NMAX,LMAX,THMAX+1),THD(THMAX),KKARAY(THMAX+1)
      DIMENSION NOBS(THMAX+1),C5(LMAX),SAX(NMAX,LMAX),CRITIC(10)
      DIMENSION AB0(LMAX),AB1(LMAX),SAXI(NMAX,LMAX),CMATX(2,LMAX)
      DIMENSION YKL(1,1),YI(1,LMAX),B3(LMAX),RI(1000),B13(LMAX,1)
      DIMENSION SIMV(NMAX),ANORM(2),DENORM(NMAX),IP1T(1),IP2T(1)
      DIMENSION IPOW1(10),IPOW2(10),TESPOW1(10),TESPOW2(10)
C
C     *** EXTERNAL SUBROUTINES ***
C
      EXTERNAL MULTMAT,MATINV,ZEROMAT,ZEROVEC,EQMAT,EQVEC
      EXTERNAL SORT,IZEROVEC,ZEROMAT3
      EXTERNAL GAP,ORDER,HUSHLD,GNORM,NFR,bar,dem,mmtl
      EXTERNAL ARMFIT,VARCOV,SINV,MRQCOF
C
C     *** SIMULATION DATA ***
      write(6,*) 'nsim?'
      read(5,*) nsim
c      NSIM=500
      N=105
      IS = 4
      IP1 = 1
      IP2 = 1
      CRITIC(1) = 10.5966
      CRITIC(2) = 9.21034
      CRITIC(3) = 7.37776
      CRITIC(4) = 5.99147
      CRITIC(5) = 4.60517
      CRITIC(6) = 0.21072
      CRITIC(7) = 0.1025870
      CRITIC(8) = 0.0506356
      CRITIC(9) = 0.0201007
      CRITIC(10) = 0.0100251
C
C     *** OPEN RESULTS FILE ***
C
      OPEN (3,FILE='sim.res',STATUS='UNKNOWN')
C
C     *** LIMITING NUMBER OF ITERATIONS
C
       I0=50
C**********************************************************************
C     SIMULATION NON MULTPLICATIVE STATIONARY AR PROCESS PARAMETERS
C
C**********************************************************************
C
C     *** INTRODUCTION OF PARAMETERS ***
C
      NPAM=5
      B3(1)=0.5
      B3(2)=0.0
      B3(3)=0.0
      B3(4)=0.5
      B3(5)=-0.25
      CALL IZEROVEC(IPOW1,10,10)
      CALL IZEROVEC(IPOW2,10,10)
      SEED=12345
      N3=N
      THD(1)=0.0
      KKARAY(1)=IP1+IS*IP2
      ITF=KKARAY(1)
      I5=ITF+1
      K1=ITF+2
      KRL=IP1+1
      IL=IP1+IP2*KRL
      I=0
      IRS=0
C     *** SIMULATION START ***
      do 250 i=1,nsim
c 250    CONTINUE
c        IRS=IRS+1
c        write(6,*) i,IRS
c          write(6,*) i
C
C     *** AR(IP1,IP2,S) MODEL GENERATION *** 
C
         I9=0
         VA=0.0
         S2=0.0
         N=N3
         CALL ZEROVEC(Y,NMAX,N)
         CALL ZEROVEC(RI,1000,1000)
         CALL GNORM(RI,1000,SEED)
         CALL ZEROMAT(B13,LMAX,NPAM,1)
	   CALL ZEROMAT(YI,1,1,NPAM)
	   YKL(1,1)=0.0
         DO 12 IT0=1,NPAM
            B13(IT0,1)=B3(IT0)
 12         YI(1,IT0)=RI(200+IT0)
            DO 15 IT3 = 1, N
               CALL MULTMAT(YI,B13,YKL,1,NMAX,1,NPAM,1)
               Y(IT3)=YKL(1,1)+RI(300+IT3)              
               DO 17 IT0=0,NPAM-2
                  IK=NPAM-IT0
 17               YI(1,IK)=YI(1,IK-1)
               YI(1,1)=Y(IT3)
 15      CONTINUE
         CALL EQVEC(Y,Y1,NMAX,N)
C*********************************************************************
C     ESTIMATION OF THE FIRST MODEL: AR - TEST OF GAPS
C     
C**********************************************************************
         IER=0
         IES=0
         IND=1
         D9=0.0
         CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
         CALL IZEROVEC(NOBS,THMAX+1,1)
         CALL ZEROMAT(SAX,NMAX,N,K1)
         CALL ZEROMAT(SAXI,NMAX,N,K1)
         CALL ZEROVEC(A0,LMAX,K1)
         CALL SORT(Y,N,AX,NMAX,LMAX,KKARAY,THD,0,0,NOBS,IER,IES)
         N1=NOBS(1)         
 1900    IF (I9.GT.0) GOTO 2300
C
C     COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE GAPS MODEL
C
         DO 26 I1=1,N1
	      DO 26 I2=1,K1
                 SAX(I1,I2)=AX(I1,I2,1)
 26              SAXI(I1,I2)=SAX(I1,I2)
         CALL HUSHLD(SAX,N1,K1,NMAX)        
         CALL ARMFIT(SAX,NMAX,AC1,I5,N1,VA,IMIN,AIC,SBIC)
         CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
         DO 27 I1=1,I5
 27         B1(I1)=AC1(I1)
         S2=VA
 2300    CONTINUE
         CALL EQVEC(B1,B0,LMAX,I5)
         CALL ZEROMAT(A,LMAX,I5,I5)
         CALL ZEROVEC(A2,LMAX,I5)
         CALL MRQCOF(SAXI,NMAX,S2,N1,B1,I5,A,A2,LMAX)
         CALL ZEROVEC(C5,LMAX,I5)
         CALL GAP(A2,C2,C5,A0,LMAX,A,Q,B,B2,
     1            ELP,B9,B11,LMAX,IP1,IP2,IS)
         D9=DABS(A0(1))
         DO 74 I1=1,K1
 74         IF(D9.LT.DABS(A0(I1))) D9=DABS(A0(I1))
c         IF (D9.GT.10.0) GOTO 250
         CALL ZEROVEC(B1,LMAX,K1)
         CALL ZEROVEC(AB0,LMAX,K1)     
         CALL ORDER(B0,AB0,LMAX,IP1,IP2,IS,0)
         DO 76 I1=1,I5
c            write(3,*) a0(i1),i1
 76         B1(I1)=AB0(I1)+A0(I1)
         S2=S2+A0(K1)
         IF (D9.LE.(1.D-3).OR.I9.EQ.I0) THEN
            DO 75 I1=1,I5
 75            CMATX(1,I1)=B1(I1)
            GOTO 2800
         END IF
         CALL ZEROVEC(AB1,LMAX,I5)
         CALL ORDER(B1,AB1,LMAX,IP1,IP2,IS,1)
         CALL EQVEC(AB1,B1,LMAX,I5)
         I9=I9+1
         GOTO 1900
 2800    CONTINUE         
         C3=0.0
         DO 90 I1=IL+2,I5
            DO 90 I2=IL+2,I5
 90            C3=C3+B1(I1)*B1(I2)*B(I1,I2)
         C3=N1*C3
         IP1T(1)=IP1
         IP2T(1)=IP2
         CALL NFR(Y1,N1,IES,THD(1),0,0,CMATX,NOBS,
     1    IP1T,IP2T,IS,SIMV,DENORM,ANORM,NMAX)
         DO 200 I1=1,N1
C     ATTENTION ONLY FOR P1=1,P2=1 & S=4!!!!
            DO 210 I2=1,6
 210           SX(I1,I2)=-AX(I1,I2,1)
 200        SX(I1,7)=SIMV(I1)
         call bar(simv,bx,n1)
         call dem(simv,bx,n1)
         call mmtl(simv,simv,n1,v)         
         CALL HUSHLD(SX,N1,7,NMAX)
         CALL ARMFIT(SX,NMAX,AC3,6,N1,VA,IMIN,AIC,SBIC)         
         CALL VARCOV(SX,NMAX,IMIN,VA,SDM)
         v=v/n1
c         write(6,*) i,v,va
         C4=1.0-(va/v)
         C4=C4*N1        
C     *** CALCULATION OF POWER ***
         DO 280 IK=1,10
            IF (C3.GT.CRITIC(IK)) IPOW1(IK)=IPOW1(IK)+1
            IF (C4.GT.CRITIC(IK)) IPOW2(IK)=IPOW2(IK)+1
 280     CONTINUE                        
 250  continue
      DO 260 IK=1,10
         TESPOW1(IK) = (DBLE(IPOW1(IK))/DBLE(NSIM))*100
         TESPOW2(IK) = (DBLE(IPOW2(IK))/DBLE(NSIM))*100
 260  CONTINUE
C
C     *** SIMULATION RESULTS ***
C    
      WRITE (3,901)
      WRITE (3,902) N1
      WRITE (3,911)  
      WRITE (3,912) B3(1)
      WRITE (3,913) B3(2)
      WRITE (3,914) B3(3)
      WRITE (3,915) B3(4)
      WRITE (3,916) B3(5)
      WRITE (3,906)
      WRITE (3,902) NSIM
      WRITE (3,907)
C      WRITE (3,910) TESPOW1
      DO 370 IK=1,10
 370     WRITE (6,910) TESPOW1(IK),TESPOW2(IK)
C      WRITE (3,909)
C      WRITE (3,910) TESPOW2
C      WRITE (3,908) TESPOW2
C
C     *** CLOSE RESULTS FILE ***
C
      CLOSE(3)
C
C     *** WRITING FORMATS ***
C
  901 FORMAT ('EFECTIVE SAMPLE SIZE:') 
  902 FORMAT ('',I5)
  903 FORMAT ('AUTOREGRESSIVE REGULAR ORDER:')
  904 FORMAT ('AUTOREGRESSIVE SEASONAL ORDER:')
  905 FORMAT ('SEASONAL PERIODICITY:')
  906 FORMAT ('NUMBER OF SIMULATIONS:')
  907 FORMAT ('SEASONAL-GAPS TEST POWER:')
  909 FORMAT ('LM TEST POWER:')
  908 FORMAT ('',I5,3x,F10.5,5x,f10.5)
  910 FORMAT ('',F10.5,'   ',F10.5)
  911 FORMAT ('SIMULATED MODEL:')
  912 FORMAT ('PHI(1): ',F10.3)
  913 FORMAT ('PHI(2): ',F10.3)
  914 FORMAT ('PHI(3): ',F10.3)
  915 FORMAT ('PHI(4): ',F10.3)
  916 FORMAT ('PHI(5): ',F10.3)
  
C
      END


      SUBROUTINE MULTMAT(AMAT,BMAT,CMAT,ID1MAX,ID2MAX,L1,L2,L3)
C**********************************************************************
C     PORPOSE: MATRIX MULTIPLICATION
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C            BMAT, MATRIX DIMENSION L2,L3
C     OUPUT: CMAT, MATRIX DIMENSION L1,L3
C**********************************************************************

      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID2MAX,*),CMAT(ID1MAX,*)
      EXTERNAL ZEROMAT

      CALL ZEROMAT(CMAT,ID1MAX,L1,L3)
     
C
      DO 99 J=1,L3
	 DO 98 K=1,L2
	    IF (BMAT(K,J).EQ.0.0) GOTO 98
	    DO 97 I=1,L1
	       CMAT(I,J)=CMAT(I,J)+AMAT(I,K)*BMAT(K,J)
   97       CONTINUE
   98    CONTINUE
   99 CONTINUE
C
      RETURN
      END

   
      SUBROUTINE MATINV(AMAT,BMAT,ELP,ID1MAX,L1,IFAIL)
C************************************************************************
C     PORPOSE: MATRIX INVERSION
C     INPUT: AMAT, MATRIX DIMENSION L1,L1
C     OUTPUT: BMAT, MATRIX DIMENSION L1,L1
C             IFAIL, MONITORING VALUE
C************************************************************************

      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*),ELP(ID1MAX,*)
      EXTERNAL EQMAT,ZEROMAT
C     
      CALL ZEROMAT(ELP,ID1MAX,L1,L1)
      CALL EQMAT(AMAT,ELP,ID1MAX,L1,L1)
      DO 115 I=1,L1
	 BMAT(I,I)=1.0
  115 CONTINUE
      DO 99 J=1,L1
	 DO 98 I=J,L1
	    IF (ELP(I,J).NE.0.0) GOTO 11
   98    CONTINUE
	 IFAIL=1
	 RETURN
   11    CONTINUE
      DO 97 K=1,L1
C
	 PIVOT01= ELP(J,K)
	 ELP(J,K)= ELP(I,K)
	 ELP(I,K)= PIVOT01
C
	 PIV= BMAT(J,K)
	 BMAT(J,K)=BMAT(I,K)
	 BMAT(I,K)= PIV
   97 CONTINUE
C
      IF(ELP(J,J).NE.0.0) GOTO 22
C
      IFAIL=1
      RETURN
C
C
   22 CONTINUE
      PPP= 1.0/ELP(J,J)
      DO 96 K=1,L1
         ELP(J,K)= ELP(J,K)*PPP
	 BMAT(J,K)= BMAT(J,K)*PPP
   96 CONTINUE
C
      DO 95 K=1,L1
C
	 IF(J.EQ.K) THEN
C
   33       CONTINUE
C
	 ELSE
	    PPP= ELP(K,J)
	    DO 94 L=1,L1
C
	       ELP(K,L)=ELP(K,L)-ELP(J,L)*PPP
	       BMAT(K,L)=BMAT(K,L)-BMAT(J,L)*PPP
   94       CONTINUE
C
	 END IF
   95 CONTINUE
   99 CONTINUE
C
      RETURN
      END

     
      SUBROUTINE ZEROMAT(AMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: CREATE A MATRIX OF ZEROS
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C     OUTPUT: AMAT, ZEROED MATRIX DIMENSION L1,L2
C**************************************************************
      
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*)
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    AMAT(I,J)=0.0
  99     CONTINUE
  98  CONTINUE
      RETURN
      END

      SUBROUTINE ZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PORPOSE: CREATE A VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION VEC(ID1MAX)

      DO 98 I=1,L1     
		VEC(I)=0.0
  98  CONTINUE
      RETURN
	END
   
      SUBROUTINE EQMAT(AMAT,BMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: EQUALIZATION OF MATRICES 
C     INPUT: AMAT, MATRIX DIMENSION L1 L2
C     OUPUT: BMAT, MATRIX EQUAL AMAT DIMENSION L1 L2
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*)
C
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    BMAT(I,J)=AMAT(I,J)
   99    CONTINUE
   98 CONTINUE
      RETURN
      END

      SUBROUTINE EQVEC(VECA,VECB,ID1MAX,L1)
C******************************************************************
C     PORPOSE: EQUALIZATION OF VECTORS
C     INPUT: VECA, VECTOR DIMENSION L1
C     OUTPUT: VECB, VECTOR EQUAL VECA DIMENSON L1
C******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VECA(ID1MAX),VECB(ID1MAX)
C
      DO 99 I=1,L1
	    VECB(I)=VECA(I)
   99 CONTINUE
      RETURN
      END

      SUBROUTINE SORT(Y,N,AX,IAX1,IAX2,KKARAY,THD,NTHD,ID,NOBS,IER,IES)
C***********************************************************************
C     PORPOSE: SORT THE DATA FOR WHICH AN APPROPIATE SAR-MODEL APPLIES
C     INPUT: Y, DATA
C            N, ITS DIMENSION
C            THD, ARRAY CONTAINING THE THRESHOLDS
C            NTHD+1, NUMBER OF PIECES
C            ID, DELAY PARAMETER
C     OUTPUT:AX, ARRAY CONTAINING THE SORTED DATA
C            IER, ERROR CODE 0 OK
C                            1 SOME PIECES DO NOT HAVE ENOUGH DATA TO
C                                    ESTIMATE THE AR-MODEL
C            IES, START POSITION OF SORTING
C
C            NOBS(I), NUMBER OF DATA IN THE I-TH PIECE
C ***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION NOBS(*),Y(*),THD(*)
      DIMENSION AX(IAX1,IAX2,1),KKARAY(*)
C     *** NUMBER OF PIECES ***
      JEND=NTHD+1
C     *** CLEANING NOBS  ***
      DO 10 I=1,JEND
 10      NOBS(I)=0

      
C     ***  CHOSE APPROPIATE STARTING POSITION  ***
      IES=ID
      DO 20 I1=1,JEND
	 KK=KKARAY(I1)
 20      IES=MAX0(IES,KK)
      IES=IES+1
C     *** SORT X INTO AX'IS ***
      DO 30 I=IES,N
	 ICP=1
C     *** ASSIGN TO FIRST PIECE FOR NTHD=0 ***
	 IF (NTHD.EQ.0) GOTO 50
C     *** FIRST ASSIGN TO LAST PIECE ***
	 ICP=NTHD+1
	   I1=I-ID      
C     *** SEE Y(IT) BELONGS TO WHICH PIECE
	 DO 40 K=1,NTHD
	    IF (Y(I1).GT.THD(K)) GOTO 40
	    ICP = K
C     *** AH, IT'IS FOUND ***
	    GOTO 50
  40     CONTINUE
  50     CONTINUE
C     *** INDICATE THE CURRENT POSITION X(I) OCCUPIES IN THE ICP PIECE
	 NOBS(ICP)=NOBS(ICP)+1
	 NOB=NOBS(ICP)
	 KEND=KKARAY(ICP)+1
C     *** ASSIGN AX(NOB,I)
	 DO 60 K=1,KEND
	   AX(NOB,K,ICP)=Y(I-K+1)
  60     CONTINUE
C     *** REASSIGN AX TO AVOID CASE KK=0
	 AX(NOB,1,ICP)=1.0
	 KEND=KEND+1
	 AX(NOB,KEND,ICP)=Y(I)
  30  CONTINUE
C     *** CHECK IF ENOUGH DATA TO ESTIMATE THE MODEL ***
      IER=0
      DO 70 J=1,JEND
	 NN=NOBS(J)
	 IF (NN.GE.20) GOTO 70
	 IER=1
C     *** The JTH REGIME LACKS DATA, SO RETURN WITH IER=1 ***
	 RETURN
  70  CONTINUE
      RETURN
      END


      SUBROUTINE GAP(A2,C2,C5,A0,ILX,A,Q,B,B2,ELP,B9,B11,IMX,IP1,
     1               IP2,IS)
C*****************************************************************
C       PORPOSE: CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS 
C              PARTIAL DERIVATIVES GAPS TESTING
C     INPUT: A2, VECTOR OF SCORES
C            A, INFORMATION MATRIX
C            IFS, FIRST ORDER GAP TESTING
C            IR, NUMBER OF ORDERS TO BE TESTED (BEGINS @ IFS)
C     OUTPUT: A0, VECTOR OF ADJUSTING FACTORS
C             B, MATRIX OF RESTRICTED ESTIMATION
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A2(*),C2(*),C5(*),A0(*)
      DIMENSION Q(IMX,*),B(IMX,*),A(IMX,*),B2(IMX,*),ELP(IMX,*)
      DIMENSION B9(IMX,*),B11(IMX,*)
      EXTERNAL ZEROVEC,EQVEC,ZEROMAT,MULTMAT,MATINV,EQMAT

C     INTRODUCTION OF SCORES
      KRP=IP1+IP2*IS
      IR=IS-IP1-1
      IL=IP1+IP2*(IP1+1)
      CALL ZEROVEC(C2,IMX,IL+IR)
      CALL ZEROVEC(C5,IMX,KRP)
      DO 10 I=1,KRP+2
 10      C2(I)=A2(I)
      C5(KRP+2)=C2(KRP+2)
C     PERMUTATION OF SCORE VECTOR
      DO 15 I1=1,IP2+1
	 DO 15 I=1,IP1+1
 15         C5(I+(I1-1)*(IP1+1))=C2(I+(I1-1)*IS) 
      DO 20 I1=1,IP2
	 DO 20 I=1,IR
 20         C5(1+IL+I+(I1-1)*IR)=C2(1+IP1+I+(I1-1)*IS)
      CALL ZEROMAT(B,IMX,KRP+1,KRP+1)
      CALL ZEROMAT(B2,IMX,KRP+1,KRP+1)
C     PERMUTATION OF THE INFORMATION MATRIX
      DO 50 I1=1,IP2+1
	 DO 50 I2=1,IP1+1
	    DO 50 I3=1,IP2+1
	       DO 50 I4=1,IP1+1
		  B(I2+(I1-1)*(IP1+1),I4+(I3-1)*(IP1+1))=
     1            A(I2+(I1-1)*IS,I4+(I3-1)*IS)
 50   CONTINUE
      DO 60 I1=1,IP2
	 DO 60 I2=1,IR
	    DO 60 I3=1,IP2
	       DO 60 I4=1,IR
		  B(1+IL+I2+(I1-1)*IR,1+IL+I4+(I3-1)*IR)=
     1            A(1+IP1+I2+(I1-1)*IR,1+IP1+I4+(I3-1)*IR)
 60   CONTINUE      
      DO 170 I1=1,KRP+2
	 B(KRP+2,I1)=A(KRP+2,I1)
 170     B(I1,KRP+2)=B(KRP+2,I1)
      CALL ZEROMAT(ELP,IMX,KRP+2,KRP+2)
      IFAIL=0
      CALL MATINV(B,B2,ELP,IMX,KRP+2,IFAIL)
      CALL EQMAT(B2,B,IMX,KRP+2,KRP+2)
c      do 172 il=1,krp+2
c 172     write(6,*) (b(il,ik),ik=1,krp+2)
      CALL ZEROMAT(B9,IMX,KRP+2,1)
      CALL ZEROMAT(B11,IMX,KRP+2,1)
      DO 190 I=1,KRP+2
 190     B9(I,1)=C5(I)
      CALL MULTMAT(B,B9,B11,IMX,IMX,KRP+2,KRP+2,1)
      CALL ZEROVEC(A0,IMX,KRP+2)
	DO 200 I=1,KRP+2
 200     A0(I)=B11(I,1)
      RETURN
      END
     
      SUBROUTINE ZEROMAT3(AMAT,ID1MAX,ID2MAX,L1,L2,L3)
C************************************************************
C
C     PORPOSE: CREATE A 3D MATRIX OF ZEROS   
C     INPUT: AMAT, MATRIX TO BE ZEROED
C            L1,L2,L3, DIMENSIONS
C     OUTPUT: AMAT, ZEROED MATRIX
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,ID2MAX,*)

      DO 97 M=1,L3
	 DO 98 J=1,L2
	    DO 99 I=1,L1
	       AMAT(I,J,M)=0.0
  99        CONTINUE
  98     CONTINUE
  97   CONTINUE
C
      RETURN
      END

      SUBROUTINE IZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PORPOSE: CREATE A INTEGER VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      INTEGER ID1MAX,L1,VEC
	DIMENSION VEC(*)

      DO 98 I=1,L1     
		VEC(I)=0
  98  CONTINUE
      RETURN
	END

      SUBROUTINE ORDER(B,C,ID1MAX,IP1,IP2,IS,IA)
C************************************************************
C     PORPOSE: PERMUTATE A VECTOR   
C     INPUT: B, VECTOR TO BE PERMUTATED
C            IO, LENGTH OF THE VECTOR
C            IIFS, FIRST POSITION OF PERMUTATION
C            IIR, ORDERS TO BE PERMUTATED
C     OUTPUT: C, VECTOR PERMUTATED
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B(*),C(*)
C     PERMUTATION IN THE VECTOR
      IR=IS-IP1-1
      IL=IP1+IP2*(IP1+1)
      IF (IA.EQ.0) THEN
	 DO 10 I1=1,IP2+1
	    DO 10 I=1,IP1+1
 10            C(I+(I1-1)*(IP1+1))=B(I+(I1-1)*IS)
	 DO 20 I1=1,IP2
	    DO 20 I=1,IR
 20            C(1+IL+I+(I1-1)*IR)=B(1+IP1+I+(I1-1)*IS)
       ELSE IF(IA.EQ.1) THEN
	 DO 30 I1=1,IP2+1
	    DO 30 I=1,IP1+1
 30          C(I+(I1-1)*IS)=B(I+(I1-1)*(IP1+1))
	 DO 40 I1=1,IP2
	    DO 40 I=1,IR
 40          C(1+IP1+I+(I1-1)*IS)=B(1+IL+I+(I1-1)*IR)
      END IF
      RETURN
      END


      SUBROUTINE HUSHLD(AH1,N,K,IA11)
C===================================================================
C Purpose: HOUSHOLDER TRANSFORMATION
C Input : AH1 = array to be transformed;
C         N = number of columns;
C         K = number of rows;
C         IA11 = number of columns of AH1 in the calling program.
C Output: AH1 = transformed array.
C==================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*),Z(350)
      TOL=1.0E-30
      NK=MIN0(N,K)
      DO 100 II=1,NK
      H=0.0
      DO 10 I=II,N
      Z(I) = AH1(I,II)
   10 H=H+Z(I)*Z(I)
      IF(H .GT. TOL) GOTO 40
      G=0.0
      GOTO 100
   40 G=SQRT(H)
      F=AH1(II,II)
      IF(F.GE.0.0) G=-G
      Z(II)=F-G
      H=H-F*G
      IF(II .EQ. K) GOTO 100
      II1=II+1
      DO 90 J=II1,NK
      S=0.0
      DO 20 I=II,N
   20 S=S+Z(I)*AH1(I,J)
      S=S/H
      DO 30 I=II,N
   30 AH1(I,J)=AH1(I,J)-Z(I)*S
   90 CONTINUE
  100 AH1(II,II)=G
      RETURN
      END

      SUBROUTINE ARMFIT(AH1,IA11,AC1,K,N,VA,IMIN,AC3,AC4)
C*******************************************************************
C Purpose: given the lags for different pieces, compute the AIC
C          for the AR-model in the Ith piece.
C Input: AH1 = housholder transformed array (so it is upper triangular);
C        IA11 = number of rows of AH1 in the calling program;
C        K = number of columns;
C        N = number of rows (number of data);
C        ISEL = indicator for information criterion.
C Output: AR(IMIN-1) = the chosen AR-model;
C         IMIN = the number parameters of the AR-model;
C         AICM = its corresponding information criterion;
C         VA = mean sum of squared errors;
C         AC1 = contains the coefficients of the AR-model.
C Here, AC2 and AC3 are working vectors.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*)
      DIMENSION AC1(*)
      K1=K+1
      K1=MIN0(N,K1)
      OSD=0.0E00
      FN=N
      OSD=OSD+AH1(K1,K1)*AH1(K1,K1)
      AC2=OSD/FN
      AC3=FN*LOG(AC2)+2*K
      AC4=FN*LOG(AC2)+K*LOG(DBLE(FN))
      VA=AC2
      IMIN=K1-1
      IF(IMIN .EQ. 1) GOTO 200
      DO 100 M=IMIN,IMIN
	 AC1(M)=AH1(M,K1)/AH1(M,M)
	 MM1=M-1
	 DO 110 II=1,MM1
	    I=M-II
            SUM=DBLE(AH1(I,K1))
	    I1=I+1
	    DO 120 J=I1,M
  120       SUM=SUM-DBLE(AC1(J))*DBLE(AH1(I,J))
  110    AC1(I)=SUM/AH1(I,I)
  100 CONTINUE
      RETURN
  200 AC1(1)=AH1(1,K1)/AH1(1,1)
      RETURN
      END

      SUBROUTINE VARCOV(SAX,IAX,IMIN,VA,SDM)
C===================================================================
C===================================================================
C PURPOSE: TO PROVIDE THE VARIANCE COVARIANCE MATRIX FOR THE
C          ESTIMATES.
C INPUT: SAX=UPPER TRIANGULAR FORM OF THE DESIGN MATRIX
C        IAX=NO. OF COLUMNS OF SAX IN THE CALLING PROGRAM
C        IMIN: SAX IS OF DIMENSION IMIN BY IMIN
C        VA=SUM OF SQUARED ERRORS
C ACTION: THE SD ESTIMATES AND THE VARIANCE COVARIANCE MATRIX IS PRINTED
C SUBROUTINES CALLED: SINV
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SAX(IAX,*),SDM(*)
      DIMENSION TA(350)
      EXTERNAL SINV
      IC=0
C     TO COMPRESS THE UPPER TRIANGULAR MATRIX SAX INTO ONE
C     DIMENSIONAL VECTOR TA
      DO 2 I2=1,IMIN
      DO 2 I1=1,I2
      IC=IC+1
    2 TA(IC)=SAX(I1,I2)
C     CALCULATE VA*INVERSE OF THE PRODUCT OF SAX TRANSPOSE AND SAX
      CALL SINV(TA,IMIN,VA)   
C     RETIREVE THE DIAGONAL AND TAKE THE SQUARE ROOT; THEY ARE
C     ESTIMATES OF STANDARD ERROR
      I=1
      IP=1
 11   SDM(I)=SQRT(TA(IP))
      I=I+1
      IP=I+IP
      IF(I .GT. IMIN) GOTO 10
      GOTO 11
  10  RETURN
      END

      SUBROUTINE SINV(A,N,VA)
C===================================================================
C===================================================================
C PURPOSE: TO PROVIDE THE PRODUCT OF THE INVERSE A'*A AND VA
C INPUT: A THE UPPER TRIANGULAR MATRIX STACKED UP ROW-WISE
C        N=DIMENSION OF THE TRIANGULAR MATRIX,SO N*(N-1)=LENGTH OF A
C        VA=MEAN SUM OF SQUARE ERROR
C OUTPUT: A=THE DESIRED PRODUCT AS DESCRIBED IN THE PURPOSE
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(350)
      IPIV=N*(N+1)/2
      IND=IPIV
      DO 6 I=1,N
      DIN=1.D0/DBLE(A(IPIV))
      A(IPIV)=DIN
      MIN=N
      KEND=I-1
      LANF=N-KEND
      IF(KEND) 5,5,2
    2 J=IND
      DO 4 K=1,KEND
      WORK=0.D0
      MIN=MIN-1
      LHOR=IPIV
      LVER=J
      DO 3 L=LANF,MIN
      LVER=LVER+1
      LHOR=LHOR+L
    3 WORK=WORK+DBLE(A(LVER)*A(LHOR))
      A(J)=-WORK*DIN
    4 J=J-MIN
    5 IPIV=IPIV-MIN
    6 IND=IND-1
      DO 8 I=1,N
      IPIV=IPIV+I
      J=IPIV
      DO 8 K=I,N
      WORK=0.D0
      LHOR=J
      DO 7 L=K,N
      LVER=LHOR+K-I
      WORK=WORK+DBLE(A(LHOR)*A(LVER))
    7 LHOR=LHOR+L
      A(J)=WORK*DBLE(VA)
    8 J=J+K
      RETURN
      END

      SUBROUTINE MRQCOF(X,LX,SIG,NDATA,A,MA,ALPHA,BETA,
     1          NALP)
C**************************************************************
C     THIS SUBROUTINE PROVIDES THE SCORES AND THE INFORMATION
C     MATRIX OF EACH REGIME.
C     INPUT: X IS THE MATRIX OF DATA EQUAL TO THE SAX MATRIX (OUTPUT OF
C            THE 'SORT' SUBROUTINE.
C            A, VECTOR OF PRELIMINAR ESTIMATORS OF PARAMETERS    
C            SIG IS THE PRELIMINAR ESTIMATOR OF SIGMA SQUARE
C            MA # OF PARAMETERS ( CONSTANT + AR ORDER ) ( WE USE MA+1 TO
C            INCLUDE SIGMA-SQUARE DERIVATIVES )
C     OUTPUT: ALPHA INFORMATION MATRIX
C             BETA VECTOR OF SCORES
C     EXTERNAL FUNCTIONS:
C            FUNCS EVALUATES THE DERIVATIVES AND SECOND DERIVATIVES @ EACH
C             POINT
C**************************************************************

      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(LX,*),ALPHA(NALP,*),XI(10),D2YDA(40,40)
     1         ,A(*),BETA(*),DYDA(10)
c      common v2
      EXTERNAL FUNCS,ZEROVEC,ZEROMAT
      CALL ZEROVEC(BETA,10,10)
      CALL ZEROMAT(ALPHA,10,10,10)
      CALL ZEROMAT(D2YDA,40,10,10)
      CALL ZEROVEC(XI,10,10)             
      DO 12 J=1,MA+1
	   DO 11 K=1,J
 11         ALPHA(J,K)=0.0
 12      BETA(J)=0.0
      PF3=0.0
      PF4=0.0
      DO 15 I=1,NDATA
	 XI(MA)=X(I,MA+1)         
	 DO 20 IQ=2,MA
 20         XI(MA+1-IQ)=X(I,IQ)
      CALL FUNCS(XI,A,YMOD,DYDA,MA,SIG,D2YDA,10,pf3)      
      DO 14 J=1,MA+1
	 DO 13 K=1,J
 13         ALPHA(J,K)=ALPHA(J,K)+D2YDA(J,K)
      BETA(J)=BETA(J)+DYDA(J)
 14   CONTINUE
      PF4=PF4+PF3
 15   CONTINUE 
      BETA(MA+1)=(-0.5*(NDATA*(SIG-(A(1)**2.0))-PF4))/(SIG**2.0)
      DO 19 L=1,MA+1
 19       BETA(L)=BETA(L)/DBLE(NDATA)
      DO 16 J=2,MA+1
	   DO 16 K=1,J-1
 16         ALPHA(K,J)=ALPHA(J,K)
      ALPHA(MA+1,MA+1)=(0.5*(NDATA*(SIG-(2*(A(1)**2.0))-
     1        PF4)))/(SIG**3.0)
      DO 18 J=1,MA+1
	 DO 18 K=1,MA+1
 18         ALPHA(J,K)=-ALPHA(J,K)/DBLE(NDATA)
      RETURN
      END

      SUBROUTINE FUNCS(X,A,Y,DYDA,NA,S2,D2YDA,NALP,pf3)
C**********************************************************************
C     THIS SUBROUTINE EVALUATES THE DERIVATIVES AND SECOND DERIVATIVES OF
C     THE LOG-LIKELIHOOD FUNCTION OF AN AR PROCESS 
C     INPUT: X, VECTOR WITH THE VALUE X(T) AND THE REGRESSORS NEEDED 
C            TO FORECAST X(T). ( X(NA)=X(T) X(NA-1)=X(T-1)...X(1)=X(T-P) )
C            A, VECTOR OF PRELIMINAR ESTIMATORS OF PARAMETERS
C            S2, SIGMA SQUARE VALUE
C     OUTPUT: DYDA, DERIVATIVE VECTOR @ X(T) 
C             D2YDA, SECOND DERIVATIVES MATRIX @ X(T)
C***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(*),DYDA(*),X(*),D2YDA(NALP,*)
      EXTERNAL ZEROVEC,ZEROMAT
      Y=0.0
      Q=0.0
      R=0.0
      PA=0.0
      PB=0.0
      PC=0.0
      PD=0.0
      PE=0.0
      PF=0.0
      PF1=0.0
      PF2=0.0
      PF3=0.0
      CALL ZEROVEC(DYDA,10,NA+1)
      CALL ZEROMAT(D2YDA,NALP,NA+1,NA+1)
      DO 1 J=2,NA
	 Q=Q+(X(NA-J+1)*A(J))
 1       R=R+((X(NA-J+1)**2.0)*(A(J)**2.0))
      PF=X(NA)-(A(1)+Q)
      pf1=x(na)-q
      pf2=pf1-2.0*a(1)
      pf3=pf1*pf2
C     *** derivative w.r.t constant ***
      DYDA(1)=PF/S2
C     *** derivatives w.r.t. AR parameters ***

      DO 2 K=2,NA
 2        DYDA(K)=(PF*X(NA-K+1))/S2

C     *** second derivative w.r.t constant ***

      D2YDA(1,1)=-1.0/S2

C     *** second derivatives w.r.t constant and parameters ***

      DO 3 L=2,NA
 3       D2YDA(L,1)=-X(NA-L+1)/S2

C     *** second derivative w.r.t constant and sigma square ***
 
      D2YDA(NA+1,1)=-PF/(S2**2.0)

C     *** second derivatives of parameters ***

      DO 4 L=2,NA
	 DO 4 K=2,L
 4          D2YDA(L,K)=-(X(NA-L+1)*X(NA-K+1))/S2

C     *** second derivative of sigma-square ***

c      D2YDA(NA+1,NA+1)=(-S2+4.0*PE)/(-2.0*(S2**3.0))

C     *** second derivative of sigma-square and parameters ***

      DO 6 L=2,NA
 6       D2YDA(NA+1,L)=-(PF*X(NA-L+1))/(S2**2.0)
      RETURN
      END

C
C
C
		subroutine gnorm(r,nr,rs)
c               IMPLICIT REAL (A-H,O-Z)
                integer nr,rs
                REAL*8 r(nr)
		do  1 i=1,nr
99		continue		
		v=randu(rs)*2.-1.
		u=randu(rs)*2.-1.
		w=v*v+u*u
		if(w.gt.1)goto 99
		c=sqrt(-2.*alog(w)/w)
		R(i)=c*v
		r(i+1)=c*u
1		continue
		return
		end				
	   	   
C**************************************************************************  
C  
      REAL FUNCTION RANDU(IX)  
C  
C UNIFORM RANDOM NUMBER GENERATOR.  
C REF: SCHRAGE, L. 1979 'A MORE PORTABLE FORTRAN RANDOM NUMBER GENERATOR'  
C      A. C. M. TRANSACTIONS ON MATHEMATICAL SOFTWARE V5 132-138.  
C  
      INTEGER A,P,IX,B15,B16,XHI,XALO,LEFTLO,FHI,K  
      DATA A/16807/,B15/32768/,B16/65536/,P/2147483647/  
      XHI=IX/B16  
      XALO=(IX-XHI*B16)*A  
      LEFTLO=XALO/B16  
      FHI=XHI*A+LEFTLO  
      K=FHI/B15  
      IX=(((XALO-LEFTLO*B16)-P)+(FHI-K*B15)*B16)+K  
      IF (IX.LT.0) IX=IX+P  
      RANDU=FLOAT(IX)*4.656612875E-10  
      RETURN  
      END  
C

      SUBROUTINE NFR(STV,N,IES,THD,NTHD,ID,CMATX,NOBS,
     1    IP1T,IP2T,IS,SIMV,DENORM,ANORM,NMAX)
C===================================================================
C===================================================================
C Purpose: to calculate the normalized fitted residuals
C          and detailed diagnostics.
C Input: STV = array of data;
C        N = its length;
C        IES = starting position from which fitted residuals are
C             calculated;
C        THD = threshold value;
C        NTHD+1 = number of pieces;
C        ID = delay;
C        CMATX(I,J) = Jth coefficient of the Ith AR-model;
C        VV(I) = the sample sum of squared errors for the Ith ar model;
C        NOBS(I) = number of data falling in the Ith piece;
C Output: SIMV(IS+1)-SIMV(N) contains the normalized residuals.
C SUBROUTINES CALLED DESACF,HIST,LINMLD,NORPLT
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION ANORM(*)
      DIMENSION STV(*),SIMV(350),CMATX(2,*),
     1      P(350),DENORM(*)
      DIMENSION NOBS(*)
      DIMENSION IP1T(*),IP2T(*)
      INTEGER INDEX(350)
c      OPEN(10,FILE='res.out',STATUS='UNKNOWN')
	BNO=0.0
      DO 878 I=1,2
  878 ANORM(I)=0.0
c      WRITE(10,100)
c  100 FORMAT(//' ORIGINAL DATA',5x,'FITTED DATA',5x,'RESIDUALS')
      DO 2 I=IES,N
C.....Sorting.
	 IP=1
	 IF (NTHD.EQ.0) GOTO 3
	 ITEST=I-ID
	 IF (STV(ITEST).GT.THD) IP=2
  3      CONTINUE
C.....Remember which piece by recording in 'INDEX'
	 INDEX(I)=IP
C.....Calculation of fitted residuals:
	 NPARM=1+IP1T(IP)+IP2T(IP)*(IP1T(IP)+1)
	 T=DBLE(CMATX(IP,1))
	 IF (NPARM.LT.2) GOTO 8
C.....Create vectors P: 
          P(2)=STV(I-1)
          P(3)=STV(I-4)
          P(4)=STV(I-5)
C.....Calculate fitted values:
	 DO 5 K=2,NPARM
    5       T=DBLE(CMATX(IP,K))*DBLE(P(K))+T           
    8       SIMV(I)=DBLE(STV(I))-T
C.....Writing original data, fitted data and residuals.
c            WRITE(10,101) STV(I),T,SIMV(I)
c  101       FORMAT(f12.4,6x,f10.4,6x,f10.4)
      
C.....Calculation of sum of square of residuals piece by piece.
C.....ANORM(I) finally stores the residual sum of squares in the Ith piece.
          AN=DBLE(SIMV(I))*DBLE(SIMV(I))
	    ANORM(IP)=AN+ANORM(IP)
    2       CONTINUE
	    IEND=NTHD+1
	    DO 17 I=1,IEND
	       FNOB=NOBS(I)
	       ANORM(I)=ANORM(I)/DBLE(FNOB)
   17          ANORM(I)=SQRT(ANORM(I))
C.....ANORM(I) stores root mean square of residuals in the Ith piece.
      DO 27 I=IES,N
	 IP=INDEX(I)
	 II=I-IES+1
C.....Normalize the fitted residuals. 
c      SIMV(II)=DBLE(SIMV(I))/ANORM(IP)
      SIMV(II)=DBLE(SIMV(I))
      DENORM(II)=ANORM(IP)
   27 CONTINUE
      RETURN
      END

      SUBROUTINE BAR(X,BX,N)
C===================================================================
C===================================================================
C Purpose: provide sample mean of data.
C Input: X = data;
C        N = number of data.
C Output: BX = sample mean.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*)
      DO 1 I=1,N         
    1    WORK=WORK+DBLE(X(I))
      WORK=WORK/DBLE(N)
      BX=WORK
      RETURN
      END

      SUBROUTINE DEM(X,BX,N)
C===================================================================
C===================================================================
C Purpose: mean deletion.
C Input: X = data;
C        N = number of data.
C Output: X = data - sample mean.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*)
      DO 1 I=1,N
    1    X(I)=X(I)-BX
      RETURN
      END

      SUBROUTINE MMTL(X,Y,N,V)
C===================================================================
C===================================================================
C Purpose: to obtain the inner product of X & Y.
C Input: X & Y = arrays of data;
C        N = their common length.
C Output: V = inner product of X & Y.
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(*),Y(*)
      WORK1=0.0
      DO 1 I=1,N
	 T1=X(I)
	 T2=Y(I)
	 WORK2=DBLE(T1)*DBLE(T2)
    1    WORK1=WORK1+WORK2
      V=WORK1
      RETURN
      END

