C*************************************************************************
C     FORTRAN77 code
C     File: kp.for
C     Coded by: Antoni Vidiella-i-Anguera
C
C     PROGRAM FOR A 2-STEP IDENTIFICATION OF SEASONAL
C     AUTOREGRESSIVE TIME SERIES MODELS
C     USING KOREISHA AND PUKKILA (KP) STRATEGY (IJF 14, 1998 483-496)
C     N      = SAMPLE SIZE OF THE SERIES
C     ID1MAX = MAX. DEGREE OF REGULAR DIFFERENCING 
C     ID2MAX = MAX. DEGREE OF SEASONAL DIFFERENCING
C     IP1MAX = MAX. ORDER OF NON.SEASONAL AR 
C     IS     = PERIOD OF SEASONALITY
C     METHOD OF ESTIMATION: LEAST SQUARES
C************************************************************************
      IMPLICIT REAL*8(A-H,O-Z)
      INTEGER THMAX,SEED
C
C     *** PARAMETER VARIABLES ***
C
      PARAMETER (LMAX=100,NMAX=1000,THMAX=1)
C
C     *** REAL VECTORS AND MATRICES ***
C      
      DIMENSION Y(NMAX),AC1(LMAX),TVAL(LMAX)
      DIMENSION AX(NMAX,LMAX,THMAX+1),THD(THMAX),KKARAY(THMAX+1)
      DIMENSION NOBS(THMAX+1),SAX(NMAX,LMAX),ANORM(THMAX+1)
      DIMENSION Y1(NMAX),SAGI(NMAX,LMAX),CMATX(THMAX+1,LMAX)
      DIMENSION SAG(NMAX,LMAX),SIMV(NMAX),Y2(NMAX)
      DIMENSION IP1T(THMAX+1),DENORM(NMAX),AIC(LMAX)
      DIMENSION SDM(LMAX),TINT(LMAX,2),SWITCH(1,2),YKL(1,1),YI(1,LMAX)
      DIMENSION B3(LMAX),RI(1200),B11(LMAX,1),SAXI(NMAX,LMAX)
      DIMENSION B1(LMAX),B0(LMAX),C5(LMAX),A0(LMAX),ZLN(THMAX+1,LMAX)
      DIMENSION H8(LMAX),IP2T(THMAX+1),D0(LMAX),A(LMAX,LMAX)
      DIMENSION A2(LMAX),C2(LMAX),Q(LMAX,LMAX),B(LMAX,LMAX)
      DIMENSION B2(LMAX,LMAX),ELP(LMAX,LMAX),B9(LMAX,1)
C
C     *** EXTERNAL SUBROUTINES ***
C
      EXTERNAL ZEROMAT,ZEROVEC,GNORM,EQMAT,PRELIM,MATINV
      EXTERNAL SORT,IZEROVEC,ZEROMAT3,NFR,NFR2
      EXTERNAL ORDER,HUSHLD,EQVEC,TEST2,MULTEST,MRQCOF
      EXTERNAL ARMFIT,VARCOV,SINV,MULTMAT

C
C     *** SIMULATION DATA ***
c     write(6,*) 'nsim?'
c      read(5,*) nsim
      NSIM=1
      N=208
      IS = 7
      IIP1 = 1
      IIP2 = 1
      IP2=1
      IP1MAX=20
      ID1MAX=2
      ID2MAX=2      
c
C     *** OPEN RESULTS FILE ***
C
      OPEN (3,FILE='kpsim.res',STATUS='UNKNOWN')
C
C     *** LIMITING NUMBER OF ITERATIONS
C
	I0=50
C**********************************************************************
C     SIMULATION NON MULTPLICATIVE STATIONARY AR PROCESS PARAMETERS
C
C**********************************************************************
C
C     *** INTRODUCTION OF PARAMETERS ***
C
      NPAM=8
      B3(1)=0.8
      B3(2)=0.0
      B3(3)=0.0
      B3(4)=0.0
      B3(5)=0.0
      B3(6)=0.0
      B3(7)=0.5
      B3(8)=-0.40
      SEED=12345
      THD(1)=0.0      
      ILOW=0
      N3=N
      ILOW2=0
      IHIGH=0
      IORDER=0
      IMULT=0
      IHIGH2=0
      IWRONG=0
      IMULT2=0
      IKL=0
C     *** SIMULATION START ***
      DO 150 I=1,NSIM
      write(6,*) i
C
C     *** AR(IP1,IP2,S) MODEL GENERATION *** 
C
         N=N3
         CALL ZEROVEC(Y,NMAX,N)
         CALL GNORM(RI,1200,SEED)
         DO 12 IT0=1,NPAM
            B11(IT0,1)=B3(IT0)
 12         YI(1,IT0)=RI(200+IT0)
            DO 15 IT3 = 1, N
               CALL MULTMAT(YI,B11,YKL,1,NMAX,1,NPAM,1)
               Y(IT3)=YKL(1,1)+RI(200+IT3)              
               DO 17 IT0=0,NPAM-2
                  IK=NPAM-IT0
 17               YI(1,IK)=YI(1,IK-1)
               YI(1,1)=Y(IT3)
 15      CONTINUE

C**********************************************************************
C     ESTIMATION OF THE FIRST MODEL: AR  2-step procedure
C     
C**********************************************************************
         IER=0
         IES=0
         THD(1)=0.0
         ICP=1
         N3=N
         DO 300 ID2=0,ID2MAX
            DO 300 ID1=0,ID1MAX
               DO 300 IP1=0,IP1MAX
                  N=N3
	            CALL EQVEC(Y,Y1,NMAX,N)
	            IF (ID1.NE.0) THEN
		         DO 301 I3=1, ID1
		            N = N-1
		            DO 301 I4=1, N
 301                       Y1(I4) = Y1(I4+1) - Y1(I4)
	            END IF
	            IF (ID2.NE.0) THEN
		         DO 303 I3=1, ID2
		            N = N - IS
		            DO 303 I4=1, N
 303                       Y1(I4) = Y1(I4+IS) - Y1(I4)
	            END IF
	            KKARAY(1)=IP1                                    
                  CALL EQVEC(Y1,Y2,NMAX,N)
	            CALL ZEROMAT3(AX,NMAX,LMAX,N,IP1MAX,THMAX+1)
                  CALL SORT(Y1,N,AX,NMAX,LMAX,KKARAY,THD,0,0,
     1            NOBS,IER,IES)
                    N1=NOBS(1)
                   CALL ZEROMAT(SAG,NMAX,N1,IP1MAX)
                   CALL ZEROVEC(AC1,LMAX,IP1)
                   DO 306 I1=1,N
                      DO 306 I2=1,IP1+2
 306                     SAG(I1,I2)=AX(I1,I2,1)         
                   CALL HUSHLD(SAG,N1,IP1+2,NMAX)      
                   CALL ARMFIT(SAG,NMAX,AC1,IP1+1,N1,VA,IMIN,SIC)
                   CALL VARCOV(SAG,NMAX,IMIN,VA,SDM)
                   DO 307 I1=1,IP1
 307                  TVAL(I1)=AC1(I1)/SDM(I1)
                   DO 308 I1=1,IP1+1
 308                  CMATX(1,I1)=AC1(I1)
                   IP1T(1)=IP1
c                   write(3,*) (cmatx(1,ik),ik=1,ip1+1)
                   CALL NFR(Y1,N1,IES,THD(1),0,ID,CMATX,NOBS,
     1              IP1T,IS,SIMV,DENORM,ANORM,NMAX)
                   CALL TEST2(SIMV,IS,N1,IND)
                   IF (IND.EQ.1) GOTO 330
 300     CONTINUE
C     ****  We have found an order under 2-RWNAR. Next step consits in
C     ****  identifying gaps ***
 330     CONTINUE
         N4=N1
         ID1OP=ID1
         ID2OP=ID2
         IPOP=IP1                         
         IF (IPOP.LT.NPAM) THEN
C     *** In this case it is an underestimation of the correct order ***
            ILOW=ILOW+1
            GOTO 150
         ELSE IF (IPOP.GT.NPAM) THEN
C     *** In this case it is an overestimation of the correct order ***
            IHIGH=IHIGH+1
            GOTO 150
         END IF
         DO 338 I1=1,IP1-1
            TINT(I1,1)=TVAL(I1+1)
 338        TINT(I1,2)=REAL(I1)
         DO 340 I1=1,IP1-2
            DO 340 I2=1,IP1-2
               IF (DABS(TINT(I2,1)).LT.DABS(TINT(I2+1,1))) THEN
                  DO 342 I3=1,2
                     SWITCH(1,I3)=TINT(I2+1,I3)
                     TINT(I2+1,I3)=TINT(I2,I3)
 342                 TINT(I2,I3)=SWITCH(1,I3)
               END IF
 340     CONTINUE
         DO 344 I1=1,N
            SAG(I1,1)=AX(I1,1,1)
            SAG(I1,2)=AX(I1,IP1+1,1)
 344        SAG(I1,3)=AX(I1,IP1+2,1)
         CALL EQMAT(SAG,SAGI,NMAX,N1,3)
         IPL=1
         ISL=0
         DO 346 I2=1,IP1-1
            IND=0
            CALL HUSHLD(SAG,N1,IPL+2,NMAX)      
            CALL ARMFIT(SAG,NMAX,AC1,IPL+1,N1,VA,IMIN,SIC1)
            CALL VARCOV(SAG,NMAX,IMIN,VA,SDM)
            CALL ZEROMAT(CMATX,THMAX+1,1,NPAM)
            CMATX(1,1)=AC1(1)
            CMATX(1,IP1+1)=AC1(IPL+1)
            DO 348 I1=1,IPL-1
 348           CMATX(1,1+INT(TINT(IPL-I1,2)))=AC1(I1+1)            
            IP1T(1)=IP1
            CALL NFR(Y1,N1,IES,THD(1),0,ID,CMATX,NOBS,
     1           IP1T,IS,SIMV,DENORM,ANORM,NMAX)
            CALL TEST2(SIMV,IS,N1,IND)
C            write(3,*) ind ,'1 acabat model gaps'
            IF (IND.EQ.1) GOTO 500
            DO 350 I1=1,N1
               DO 350 I3=0,ISL+1
                  I6=ISL+2-I3
 350              SAG(I1,I6+2)=SAGI(I1,I6+1)
            DO 366 I1=1,N1
               SAG(I1,1)=1.0
 366           SAG(I1,2)=AX(I1,INT(TINT(ISL+1,2))+1,1)
            CALL EQMAT(SAG,SAGI,NMAX,N1,ISL+4)
            IPL=IPL+1
            ISL=ISL+1
 346     CONTINUE
 500     CONTINUE
         IF((I2-1).GT.2) THEN
            IHIGH2=IHIGH2+1
            GOTO 150
         END IF
         IF((I2-1).LT.2) THEN
            ILOW2=ILOW2+1
            GOTO 150
         END IF
         IF ((INT(TINT(1,2)).EQ.1.OR.INT(TINT(1,2)).EQ.7).OR.
     1   (INT(TINT(2,2)).EQ.1.OR.INT(TINT(2,2)).EQ.7)) THEN
            IKL=1
c            write(3,*) sic1,n1,' gaps model '
         ELSE
            IWRONG=IWRONG+1
            GOTO 150
         END IF
c         write(3,*) (INT(TINT(ik,2)),ik=1,i2-1)
       
C
C     *** IDENTIFICATION OF THE MULTIPLICATIVE ESTRUCTURE ***
C
         IF(I2-1.GT.0.AND.IPOP.GE.IS) THEN
            N=N1
            IER=0
            IES=0
            CALL ZEROMAT3(AX,NMAX,LMAX,N,LMAX,THMAX+1)
            CALL IZEROVEC(KKARAY,THMAX+1,2)
            KKARAY(1)=(2*IS)-1
            ITF=KKARAY(1)
            ID=0
            THD(1)=0.0
            CALL SORT(Y2,N,AX,NMAX,LMAX,KKARAY,THD,0,ID,NOBS,IER,IES)
            N1=NOBS(1)
            DO 800 IP1=1,IS-1            
               S2=0.0
               I5=IP1+IP2*(IP1+1)
               I9=0
C	  
C       COMPUTATION OF THE SCORE VECTOR AND INFORMATION MATRIX OF THE
C       SEASONAL MODEL 
C
 600           IF (I9.GT.0) GOTO 730
C
C       COMPUTATION OF THE ESTIMATES AND SDT ERRORS OF THE SEASONAL MODEL
C               
               K2=IP1+IP2*(IP1+1)+1
               K1=K2+1
               CALL ZEROVEC(B1,LMAX,K2)          
               CALL ZEROMAT(SAX,NMAX,N1,K1)
               CALL ZEROMAT(SAXI,NMAX,N1,K1)
               CALL ZEROVEC(SDM,LMAX,K2)
               DO 610 I1=1,N1
                  DO 612 I2=1,IP1+1
 612                 SAX(I1,I2)=AX(I1,I2,1)
                  DO 614 I3=1,IP2
                     DO 614 I4=1,IP1+1
 614                 SAX(I1,1+IP1+I4+(I3-1)*(IP1+1))=AX(I1,I4+IS*I3,1)
 610              SAX(I1,IP1+IP2*(IP1+1)+2)=AX(I1,ITF+2,1)
               CALL EQMAT(SAX,SAXI,NMAX,N1,K1)
               CALL HUSHLD(SAX,N1,K1,NMAX)
               CALL ARMFIT(SAX,NMAX,AC1,K2,N1,VA,IMIN,SIC2)
c               write(3,*) (ac1(ik),ik=1,k2)
               CALL VARCOV(SAX,NMAX,IMIN,VA,SDM)
               DO 616 I1=1,K2
 616              B1(I1)=AC1(I1)
               S2=VA 
               CALL EQVEC(B1,B0,LMAX,K2)
C     *** INTRODUCTION PRELIMINAR PARAMETERS ***
               CALL ZEROVEC(C5,LMAX,I5+2)
               CALL PRELIM(B0,D0,C5,LMAX,B1,LMAX,IP1,IP2,IS)
               do 725 i1=ip1+ip2+2,i5+1
 725              b0(i1)=-b0(i1)
               CALL EQVEC(B0,B1,LMAX,I5+1)
 730           CONTINUE
               CALL ZEROVEC(A0,LMAX,K2)
               CALL MRQCOF(SAXI,NMAX,S2,N1,B1,K2,A,A2,LMAX)
c                do 728 ik=1,k2+1
c 728             write(3,*) (a(ik,iq),iq=1,k2+1)
                
C
C         MULTIPLICATIVE RESTRICTIONS
C
               CALL MULTEST(A2,D0,C2,A0,LMAX,A,Q,B,B2,ELP,B9,B11,LMAX,
     1                   IP1,IP2,IS,i9)
               D9=ABS(A0(1))
               DO 618 I6=2,I5+1
 618              IF (D9.LT.ABS(A0(I6))) D9=ABS(A0(I6))
               CALL ZEROVEC(B1,LMAX,I5+1)
               DO 620 I6=1,I5+1
 620              B1(I6)=B0(I6)+A0(I6)
 620           write(3,*) b1(i6),b0(i6),a0(i6),i6,i9
               S2=S2+A0(I5+2)               
               IF (D9.LT.(0.001).OR.I9.EQ.I0) GOTO 780
               CALL EQVEC(B1,B0,LMAX,I5+1)
c              write(3,*) (b0(ik),ik=1,i5+1)
               DO 622 I6=1,IP1+1
 622              D0(I6)=B0(I6)
               DO 624 I1=1,IP2
                  DO 624 I2=1,IP1+1
 624                 D0(IS*I1+I2)=B0(IP1+I2+(I1-1)*(IP1+1)+1)
c               write(3,*) (d0(ik),ik=1,1+(ip2*is)+ip1)
               I9=I9+1
               GOTO 600
 780           CONTINUE
               DO 626 I2=1,I5+1              
 626              ZLN(1,I2)=B1(I2)
               AIC(IP1)=N1*LOG(S2)+(I5+1)*LOG(REAL(N1))
c               write(3,*) (b1(ik),ik=1,i5+1)
               DO 628 I6=1,IP1+1
 628              ZLN(1,I6)=ZLN(1,I6)
               DO 630 I6=1,IP2
 630              ZLN(1,IP1+I6+1)=ZLN(1,(I6-1)*(IP1+1)+IP1+2)
               DO 632 I6=1,IP1+1
 632              CMATX(1,I6)=ZLN(1,I6)
               CALL ZEROVEC(H8,LMAX,IP1+IP2+1)
               H8(1)=1.0
               DO 634 I6=1,IP1+IP2
 634              H8(I6+1)=ZLN(1,I6+1)
               DO 636 I1=1,IP2
               okl=1.0
                  DO 636 I2=1,IP1+1
                     if(i2.gt.1) okl=-1.0
 636                 CMATX(1,IP1+(IP1+1)*(I1-1)+I2+1)=
     1                 H8(I2)*H8(IP1+1+I1)*okl
c               write(3,*) (cmatx(1,ik),ik=1,i5+1)
               CALL ZEROVEC(DENORM,NMAX,NMAX)
               CALL ZEROVEC(ANORM,2,2)
               IP1T(1)=IP1
               IP2T(1)=IP2              
               CALL NFR2(Y2,N1,IES,THD(1),0,0,CMATX,NOBS,
     1         IP1T,IP2T,IS,SIMV,DENORM,ANORM,NMAX)
               CALL TEST2(SIMV,IS,N1,IND)
c               write(3,*) ind,'ind multiplicative'
               IF (IND.EQ.0) AIC(IP1)=AIC(IP1)+10.E+30
 800        CONTINUE
            IOL=1
            AL=AIC(1)
            DO 850 I1=2,IS-1
               IF(AIC(I1).LT.AL) THEN
                  AL=AIC(I1)
                  IOL=I1
               END IF
 850        CONTINUE
            IF (AIC(IOL).LT.SIC1.AND.IKL.EQ.1) THEN
               IF (IOL.EQ.1) THEN
                  IMULT=IMULT+1
               ELSE
                  IMULT2=IMULT2+1
               END IF
            ELSE
               IORDER=IORDER+1
            END IF
c            write(6,*) aic(iol),sic1,IKL
         END IF
 150  CONTINUE
      write(6,*) ILOW,ILOW2,IHIGH,IHIGH2,IORDER,IMULT,IMULT2,IWRONG
      WRITE(3,901)
      WRITE(3,902) N4
      WRITE(3,903)
      WRITE(3,902) IIP1
      WRITE(3,904)
      WRITE(3,902) IIP2
      WRITE(3,905)
      WRITE(3,902) IS
      WRITE(3,906)
      WRITE(3,902) NSIM
	WRITE(3,911)
      WRITE(3,912) B3(1)
      WRITE(3,913) B3(2)
      WRITE(3,914) B3(3)
      WRITE(3,915) B3(4)
      WRITE(3,916) B3(5)
      WRITE(3,917) B3(6)
      WRITE(3,918) B3(7)
      WRITE(3,919) B3(8)
      WRITE(3,9000)
      WRITE(3,9001) ILOW
      WRITE(3,9002)
      WRITE(3,9001) ILOW2
      WRITE(3,9003)
      WRITE(3,9001) IHIGH
      WRITE(3,9004)
      WRITE(3,9001) IHIGH2
      WRITE(3,9005)
      WRITE(3,9001) IORDER
      WRITE(3,9006)
      WRITE(3,9001) IMULT
      WRITE(3,9007)
      WRITE(3,9001) IMULT2
      WRITE(3,9008)
      WRITE(3,9001) IWRONG
 9000 FORMAT ('UNDERESTIMATION I: ESTIMATED ORDER < 8')
 9001 FORMAT (' ',I4)
 9002 FORMAT ('UNDERESTIMATION II: ESTIMATED GAP > SIMULATED GAP')
 9003 FORMAT ('OVERESTIMATION I: ESTIMATED ORDER > 8')
 9004 FORMAT ('OVERESTIMATION II: ESTIMATED GAP < SIMULATED GAP')
 9005 FORMAT ('CORRECT ORDER ( as defined in K&P )')
 9006 FORMAT ('CORRECT MODEL ( as defined in K&P )')
 9007 FORMAT ('MULTIPLICATIVE ESTIMATED NONSEASONAL ORDER > SIMULATED')
 9008 FORMAT ('ESTIMATED GAP MODEL WITH WRONG ORDERS')
  901 FORMAT ('EFECTIVE SAMPLE SIZE:') 
  902 FORMAT ('',I5)
  903 FORMAT ('AUTOREGRESSIVE REGULAR ORDER:')
  904 FORMAT ('AUTOREGRESSIVE SEASONAL ORDER:')
  905 FORMAT ('SEASONAL PERIODICITY:')
  906 FORMAT ('NUMBER OF SIMULATIONS:')
  911 FORMAT ('SIMULATED MODEL:')
  912 FORMAT ('PHI(1): ',F10.3)
  913 FORMAT ('PHI(2): ',F10.3)
  914 FORMAT ('PHI(3): ',F10.3)
  915 FORMAT ('PHI(4): ',F10.3)
  916 FORMAT ('PHI(5): ',F10.3)
  917 FORMAT ('PHI(6): ',F10.3)
  918 FORMAT ('PHI(7): ',F10.3)
  919 FORMAT ('PHI(8): ',F10.3)

      END

      SUBROUTINE ZEROMAT(AMAT,ID1MAX,L1,L2)
C**************************************************************
C     PURPOSE: CREATE A MATRIX OF ZEROS
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C     OUTPUT: AMAT, ZEROED MATRIX DIMENSION L1,L2
C**************************************************************      
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*)
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    AMAT(I,J)=0.0
  99     CONTINUE
  98  CONTINUE
      RETURN
      END

      SUBROUTINE ZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PURPOSE: CREATE A VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VEC(ID1MAX)
      DO 98 I=1,L1     
         VEC(I)=0.0
  98  CONTINUE
      RETURN
      END   

      SUBROUTINE EQVEC(VECA,VECB,ID1MAX,L1)
C******************************************************************
C     PURPOSE: EQUALIZATION OF VECTORS
C     INPUT: VECA, VECTOR DIMENSION L1
C     OUTPUT: VECB, VECTOR EQUAL VECA DIMENSON L1
C******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION VECA(ID1MAX),VECB(ID1MAX)
C
      DO 99 I=1,L1
         VECB(I)=VECA(I)
   99 CONTINUE
      RETURN
      END

      SUBROUTINE SORT(Y,N,AX,IAX1,IAX2,KKARAY,THD,NTHD,ID,NOBS,IER,IES)
C***********************************************************************
C     PURPOSE: SORT THE DATA FOR WHICH AN APPROPIATE SAR-MODEL APPLIES
C     INPUT: Y, DATA
C            N, ITS DIMENSION
C            THD, ARRAY CONTAINING THE THRESHOLDS
C            NTHD+1, NUMBER OF PIECES
C            ID, DELAY PARAMETER
C     OUTPUT:AX, ARRAY CONTAINING THE SORTED DATA
C            IER, ERROR CODE 0 OK
C                            1 SOME PIECES DO NOT HAVE ENOUGH DATA TO
C                                    ESTIMATE THE AR-MODEL
C            IES, START POSITION OF SORTING
C
C            NOBS(I), NUMBER OF DATA IN THE I-TH PIECE
C ***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION NOBS(*),Y(*),THD(*)
      DIMENSION AX(IAX1,IAX2,1),KKARAY(*)
C     *** NUMBER OF PIECES ***
      JEND=NTHD+1
C     *** CLEANING NOBS  ***
      DO 10 I=1,JEND
 10      NOBS(I)=0

      
C     ***  CHOSE APPROPIATE STARTING POSITION  ***
      IES=ID
      DO 20 I1=1,JEND
	 KK=KKARAY(I1)
 20      IES=MAX0(IES,KK)
      IES=IES+1
C     *** SORT X INTO AX'IS ***
      DO 30 I=IES,N
	 ICP=1
C     *** ASSIGN TO FIRST PIECE FOR NTHD=0 ***
	 IF (NTHD.EQ.0) GOTO 50
C     *** FIRST ASSIGN TO LAST PIECE ***
	 ICP=NTHD+1
         I1=I-ID      
C     *** SEE Y(IT) BELONGS TO WHICH PIECE
	 DO 40 K=1,NTHD
	    IF (Y(I1).GT.THD(K)) GOTO 40
	    ICP = K
C     *** AH, IT'IS FOUND ***
	    GOTO 50
  40     CONTINUE
  50     CONTINUE
C     *** INDICATE THE CURRENT POSITION X(I) OCCUPIES IN THE ICP PIECE
	 NOBS(ICP)=NOBS(ICP)+1
	 NOB=NOBS(ICP)
	 KEND=KKARAY(ICP)+1
C     *** ASSIGN AX(NOB,I)
	 DO 60 K=1,KEND
            AX(NOB,K,ICP)=Y(I-K+1)
  60     CONTINUE
C     *** REASSIGN AX TO AVOID CASE KK=0
	 AX(NOB,1,ICP)=1.0
	 KEND=KEND+1
	 AX(NOB,KEND,ICP)=Y(I)
  30  CONTINUE
C     *** CHECK IF ENOUGH DATA TO ESTIMATE THE MODEL ***
      IER=0
      DO 70 J=1,JEND
	 NN=NOBS(J)
	 IF (NN.GE.60) GOTO 70
	 IER=1
C     *** The JTH REGIME LACKS DATA, SO RETURN WITH IER=1 ***
	 RETURN
  70  CONTINUE
      RETURN
      END

C     
      SUBROUTINE ZEROMAT3(AMAT,ID1MAX,ID2MAX,L1,L2,L3)
C************************************************************
C
C     PURPOSE: CREATE A 3D MATRIX OF ZEROS   
C     INPUT: AMAT, MATRIX TO BE ZEROED
C            L1,L2,L3, DIMENSIONS
C     OUTPUT: AMAT, ZEROED MATRIX
C************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,ID2MAX,*)
      DO 97 M=1,L3
	 DO 98 J=1,L2
	    DO 99 I=1,L1
	       AMAT(I,J,M)=0.0
  99        CONTINUE
  98     CONTINUE
  97   CONTINUE
C
      RETURN
      END

      SUBROUTINE IZEROVEC(VEC,ID1MAX,L1)
C**************************************************************
C     PURPOSE: CREATE A INTEGER VECTOR OF ZEROS      
C     INPUT: VEC, VECTOR DIMENSION L1
C     OUTPUT: VEC, ZEROED VECTOR DIMENSION L1
C**************************************************************
      INTEGER ID1MAX,L1,VEC
      DIMENSION VEC(*)
      DO 98 I=1,L1     
         VEC(I)=0
  98  CONTINUE
      RETURN
      END

      SUBROUTINE HUSHLD(AH1,N,K,IA11)
C*****************************************************************
C Purpose: HOUSHOLDER TRANSFORMATION
C Input : AH1 = array to be transformed;
C         N = number of columns;
C         K = number of rows;
C         IA11 = number of columns of AH1 in the calling program.
C Output: AH1 = transformed array.
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*),Z(1000)
      TOL=1.0E-30
      NK=MIN0(N,K)
      DO 100 II=1,NK
         H=0.0
         DO 10 I=II,N
            Z(I) = AH1(I,II)
   10       H=H+Z(I)*Z(I)
         IF(H .GT. TOL) GOTO 40
         G=0.0
         GOTO 100
   40    G=SQRT(H)
         F=AH1(II,II)
         IF(F.GE.0.0) G=-G
         Z(II)=F-G
         H=H-F*G
         IF(II .EQ. K) GOTO 100
         II1=II+1
         DO 90 J=II1,NK
         S=0.0
         DO 20 I=II,N
   20       S=S+Z(I)*AH1(I,J)
         S=S/H
         DO 30 I=II,N
   30       AH1(I,J)=AH1(I,J)-Z(I)*S
   90    CONTINUE
  100    AH1(II,II)=G
      RETURN
      END

      SUBROUTINE ARMFIT(AH1,IA11,AC1,K,N,VA,IMIN,AC3)
C*******************************************************************
C Purpose: given the lags for different pieces, compute the AIC
C          for the AR-model in the Ith piece.
C Input: AH1 = housholder transformed array (so it is upper triangular);
C        IA11 = number of rows of AH1 in the calling program;
C        K = number of columns;
C        N = number of rows (number of data);
C        ISEL = indicator for information criterion.
C Output: AR(IMIN-1) = the chosen AR-model;
C         IMIN = the number parameters of the AR-model;
C         AICM = its corresponding information criterion;
C         VA = mean sum of squared errors;
C         AC1 = contains the coefficients of the AR-model.
C Here, AC3 is SIC 
C===================================================================
C===================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AH1(IA11,*)
      DIMENSION AC1(*)
      K1=K+1
      K1=MIN0(N,K1)
      OSD=0.0E00
      FN=N
      OSD=OSD+AH1(K1,K1)*AH1(K1,K1)
      AC2=OSD/FN
      AC3=FN*LOG(AC2)+K*LOG(REAL(FN))
      VA=AC2
      IMIN=K1-1
      IF(IMIN .EQ. 1) GOTO 200
      DO 100 M=IMIN,IMIN
	 AC1(M)=AH1(M,K1)/AH1(M,M)
	 MM1=M-1
	 DO 110 II=1,MM1
	    I=M-II
	    SUM=DBLE(AH1(I,K1))
	    I1=I+1
	    DO 120 J=I1,M
  120       SUM=SUM-DBLE(AC1(J))*DBLE(AH1(I,J))
  110    AC1(I)=SUM/AH1(I,I)
  100 CONTINUE
      RETURN
  200 AC1(1)=AH1(1,K1)/AH1(1,1)
      RETURN
      END

      SUBROUTINE VARCOV(SAX,IAX,IMIN,VA,SDM)
C*****************************************************************
C PURPOSE: TO PROVIDE THE VARIANCE COVARIANCE MATRIX FOR THE
C          ESTIMATES.
C INPUT: SAX=UPPER TRIANGULAR FORM OF THE DESIGN MATRIX
C        IAX=NO. OF COLUMNS OF SAX IN THE CALLING PROGRAM
C        IMIN: SAX IS OF DIMENSION IMIN BY IMIN
C        VA=SUM OF SQUARED ERRORS
C ACTION: THE SD ESTIMATES AND THE VARIANCE COVARIANCE MATRIX IS PRINTED
C SUBROUTINES CALLED: SINV
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SAX(IAX,*),SDM(*)
      DIMENSION TA(1000)
      EXTERNAL SINV
      IC=0
C     TO COMPRESS THE UPPER TRIANGULAR MATRIX SAX INTO ONE
C     DIMENSIONAL VECTOR TA
      DO 2 I2=1,IMIN
         DO 2 I1=1,I2
            IC=IC+1
    2 TA(IC)=SAX(I1,I2)
C     CALCULATE VA*INVERSE OF THE PRODUCT OF SAX TRANSPOSE AND SAX
      CALL SINV(TA,IMIN,VA)   
C     RETIREVE THE DIAGONAL AND TAKE THE SQUARE ROOT; THEY ARE
C     ESTIMATES OF STANDARD ERROR
      I=1
      IP=1
 11   SDM(I)=SQRT(TA(IP))
      I=I+1
      IP=I+IP
      IF(I .GT. IMIN) GOTO 10
      GOTO 11
  10  RETURN
      END

      SUBROUTINE SINV(A,N,VA)
C********************************************************************
C PURPOSE: TO PROVIDE THE PRODUCT OF THE INVERSE A'*A AND VA
C INPUT: A THE UPPER TRIANGULAR MATRIX STACKED UP ROW-WISE
C        N=DIMENSION OF THE TRIANGULAR MATRIX,SO N*(N-1)=LENGTH OF A
C        VA=MEAN SUM OF SQUARE ERROR
C OUTPUT: A=THE DESIRED PRODUCT AS DESCRIBED IN THE PURPOSE
C********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1000)
      IPIV=N*(N+1)/2
      IND=IPIV
      DO 6 I=1,N
      DIN=1.D0/DBLE(A(IPIV))
      A(IPIV)=DIN
      MIN=N
      KEND=I-1
      LANF=N-KEND
      IF(KEND) 5,5,2
    2 J=IND
      DO 4 K=1,KEND
      WORK=0.D0
      MIN=MIN-1
      LHOR=IPIV
      LVER=J
      DO 3 L=LANF,MIN
      LVER=LVER+1
      LHOR=LHOR+L
    3 WORK=WORK+DBLE(A(LVER)*A(LHOR))
      A(J)=-WORK*DIN
    4 J=J-MIN
    5 IPIV=IPIV-MIN
    6 IND=IND-1
      DO 8 I=1,N
      IPIV=IPIV+I
      J=IPIV
      DO 8 K=I,N
      WORK=0.D0
      LHOR=J
      DO 7 L=K,N
      LVER=LHOR+K-I
      WORK=WORK+DBLE(A(LHOR)*A(LVER))
    7 LHOR=LHOR+L
      A(J)=WORK*DBLE(VA)
    8 J=J+K
      RETURN
      END

      SUBROUTINE MULTMAT(AMAT,BMAT,CMAT,ID1MAX,ID2MAX,L1,L2,L3)
C**********************************************************************
C     PORPOSE: MATRIX MULTIPLICATION
C     INPUT: AMAT, MATRIX DIMENSION L1,L2
C            BMAT, MATRIX DIMENSION L2,L3
C     OUPUT: CMAT, MATRIX DIMENSION L1,L3
C**********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID2MAX,*),CMAT(ID1MAX,*)
      EXTERNAL ZEROMAT
      CALL ZEROMAT(CMAT,ID1MAX,L1,L3)
      DO 99 J=1,L3
	 DO 98 K=1,L2
	    IF (BMAT(K,J).EQ.0.0) GOTO 98
	    DO 97 I=1,L1
	       CMAT(I,J)=CMAT(I,J)+AMAT(I,K)*BMAT(K,J)
   97       CONTINUE
   98    CONTINUE
   99 CONTINUE
      RETURN
      END

      SUBROUTINE NFR(STV,N,IES,THD,NTHD,ID,CMATX,NOBS,
     1    IP1,IS,SIMV,DENORM,ANORM,NMAX)
C********************************************************************
C Purpose: to calculate the normalized fitted residuals
C          and detailed diagnostics.
C Input: STV = array of data;
C        N = its length;
C        IES = starting position from which fitted residuals are
C             calculated;
C        THD = threshold value;
C        NTHD+1 = number of pieces;
C        ID = delay;
C        CMATX(I,J) = Jth coefficient of the Ith AR-model;
C        VV(I) = the sample sum of squared errors for the Ith ar model;
C        NOBS(I) = number of data falling in the Ith piece;
C Output: SIMV(IS+1)-SIMV(N) contains the normalized residuals.
C SUBROUTINES CALLED DESACF,HIST,LINMLD,NORPLT
C********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION ANORM(*)
      DIMENSION STV(*),SIMV(1000),CMATX(2,*),
     1      P(1000),DENORM(*)
      DIMENSION NOBS(*)
      DIMENSION IP1(*)
      INTEGER INDEX(1000)
      BNO=0.0
      DO 878 I=1,2
  878    ANORM(I)=0.0
      DO 2 I=IES,N
C.....Sorting.
      IP=1
      IF (NTHD.EQ.0) GOTO 3
      ITEST=I-ID
      IF (STV(ITEST).GT.THD) IP=2
  3   CONTINUE
C.....Remember which piece by recording in 'INDEX'
      INDEX(I)=IP
C.....Calculation of fitted residuals:
      NPARM=1+IP1(IP)
      T=DBLE(CMATX(IP,1))
      IF (NPARM.LT.2) GOTO 8
C.....Create vectors P: 
      IF (IP1(IP).GT.0) THEN
         DO 10 I2=1,IP1(IP)
 10         P(1+I2)=STV(I-I2)  
      END IF
C.....Calculate fitted values:
      DO 5 K=2,NPARM
  5      T=DBLE(CMATX(IP,K))*DBLE(P(K))+T
  8   SIMV(I)=DBLE(STV(I))-T
c      write(3,*) i,stv(i),t,simv(i)
C.....Calculation of sum of square of residuals piece by piece.
C.....ANORM(I) finally stores the residual sum of squares in the Ith piece.
      ANORM(IP)=ANORM(IP)+DBLE(SIMV(I))*DBLE(SIMV(I))
  2   CONTINUE
      IEND=NTHD+1
      DO 17 I=1,IEND
         FNOB=NOBS(I)
         ANORM(I)=ANORM(I)/DBLE(FNOB)
   17 ANORM(I)=SQRT(ANORM(I))
C.....ANORM(I) stores root mean square of residuals in the Ith piece.
      DO 27 I=IES,N
	 IP=INDEX(I)
	 II=I-IES+1
C.....Normalize the fitted residuals.
c      SIMV(II)=DBLE(SIMV(I))/ANORM(IP)
      simv(ii)=simv(i)
      DENORM(II)=ANORM(IP)
   27 CONTINUE
      RETURN
      END

      SUBROUTINE TEST2(SIMV,IS,N,IND)
C********************************************************************
C Purpose: to calculate the 2-step white noise testing
C Input: SIMV = array of data;
C        N = its 1st length;
C        IS = order of seasonallity
C Output:IND = indicator number;
C              if IND = 0 white noise hypothesis is rejected
C              if IND = 0 white noise hypothesis is not rejected
C SUBROUTINES ZEROMAT3,SORT,ZEROMAT,HUSHLD,ARMFIT,VARCOV,ZEROVEC
C********************************************************************
       IMPLICIT REAL*8 (A-H,O-Z)
       PARAMETER (NM=1000,LM=100,NTM=2)
       DIMENSION AX(NM,LM,NTM),SIMV(*),KK(LM),THD(NTM),NOBS(NTM)
       DIMENSION SAG(NM,LM),SSIC(0:LM),AC3(LM)
       EXTERNAL ZEROMAT3,SORT,ZEROMAT,HUSHLD,ARMFIT,VARCOV,ZEROVEC
C
       KCS=INT(SQRT(REAL(N))/IS)+1
       KS=INT(SQRT(REAL(N)))
       CALL ZEROMAT3(AX,NM,LM,N,LM,NTM)
       KK(1)=KCS*IS
       THD(1)=0.0
       CALL SORT(SIMV,N,AX,NM,LM,KK,THD,0,1,NOBS,IER,IES)
C *** 2-step WHITE NOISE TESTING ***
       CALL ZEROMAT(SAG,NM,N,KCS*IS)
       N1=NOBS(1)
       DO 30 I1=0,KCS
          DO 32 I2=1,N1
             DO 34 I3=1,I1+1
 34             SAG(I2,I3)=AX(I2,1+(I3-1)*IS,1)
 32          SAG(I2,I1+2)=AX(I2,KCS*IS+2,1)
          CALL HUSHLD(SAG,N1,I1+2,NM)
          CALL ARMFIT(SAG,NM,AC3,I1+1,N1,VA,IMIN,SIC)
 30       SSIC(I1)=SIC
       SMIN=SSIC(0)
       IOP=0
       DO 36 I1=1,KCS
c         write(3,*) ssic(i1)
          IF (SSIC(I1).LT.SMIN) THEN 
             SMIN=SSIC(I1)
             IOP=I1  
          END IF
 36    CONTINUE
       IF (IOP.GT.0) GOTO 50
       CALL ZEROVEC(AC3,LM,KCS+1)
       CALL ZEROMAT(SAG,NM,N1,KCS*IS)
       CALL ZEROVEC(SSIC,LM+1,KCS+1)
       DO 38 I1=0,KS
          DO 40 I2=1,N1
             DO 42 I3=1,I1+1
 42             SAG(I2,I3)=AX(I2,I3,1)
 40             SAG(I2,I1+2)=AX(I2,KCS*IS+2,1)
c        write(3,*) (sag(1,ik),ik=1,i1+2)
          CALL HUSHLD(SAG,N1,I1+2,NM)
          CALL ARMFIT(SAG,NM,AC3,I1+1,N1,VA,IMIN,SIC)
c        write(3,*) (ac3(ik),ik=1,i1+1)
 38       SSIC(I1)=SIC
       SMIN=SSIC(0)
       IOP=0
c       write(3,*) ssic(0)
       DO 44 I1=1,KS
c        write(3,*) ssic(i1),i1
          IF (SSIC(I1).LT.SMIN) THEN 
             SMIN=SSIC(I1)
             IOP=I1  
          END IF
 44    CONTINUE
c       write(3,*) iop
       IF (IOP.EQ.0) IND=1
       GOTO 60
 50    CONTINUE
       IND=0
 60    CONTINUE
       RETURN
       END

       subroutine gnorm(r,nr,rs)
c      IMPLICIT REAL (A-H,O-Z)
       integer nr,rs
       REAL*8 r(nr)
       do  1 i=1,nr
99        continue         
          v=randu(rs)*2.-1.
          u=randu(rs)*2.-1.
          w=v*v+u*u
          if(w.gt.1)goto 99
          c=sqrt(-2.*alog(w)/w)
          R(i)=c*v
          r(i+1)=c*u
1      continue
       return
       end                      
	   	   
C**************************************************************************  
C  
      REAL FUNCTION RANDU(IX)  
C  
C UNIFORM RANDOM NUMBER GENERATOR.  
C REF: SCHRAGE, L. 1979 'A MORE PORTABLE FORTRAN RANDOM NUMBER GENERATOR'  
C      A. C. M. TRANSACTIONS ON MATHEMATICAL SOFTWARE V5 132-138.  
C  
      INTEGER A,P,IX,B15,B16,XHI,XALO,LEFTLO,FHI,K  
      DATA A/16807/,B15/32768/,B16/65536/,P/2147483647/  
      XHI=IX/B16  
      XALO=(IX-XHI*B16)*A  
      LEFTLO=XALO/B16  
      FHI=XHI*A+LEFTLO  
      K=FHI/B15  
      IX=(((XALO-LEFTLO*B16)-P)+(FHI-K*B15)*B16)+K  
      IF (IX.LT.0) IX=IX+P  
      RANDU=FLOAT(IX)*4.656612875E-10  
      RETURN  
      END  
C

      SUBROUTINE EQMAT(AMAT,BMAT,ID1MAX,L1,L2)
C**************************************************************
C     PORPOSE: EQUALIZATION OF MATRICES 
C     INPUT: AMAT, MATRIX DIMENSION L1 L2
C     OUPUT: BMAT, MATRIX EQUAL AMAT DIMENSION L1 L2
C**************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*)
C
      DO 98 J=1,L2
	 DO 99 I=1,L1
	    BMAT(I,J)=AMAT(I,J)
   99    CONTINUE
   98 CONTINUE
      RETURN
      END

      SUBROUTINE NFR2(STV,N,IES,THD,NTHD,ID,CMATX,NOBS,
     1    IP1T,IP2T,IS,SIMV,DENORM,ANORM,NMAX)
C********************************************************************
C Purpose: to calculate the normalized fitted residuals
C          and detailed diagnostics.
C Input: STV = array of data;
C        N = its length;
C        IES = starting position from which fitted residuals are
C             calculated;
C        THD = threshold value;
C        NTHD+1 = number of pieces;
C        ID = delay;
C        CMATX(I,J) = Jth coefficient of the Ith AR-model;
C        VV(I) = the sample sum of squared errors for the Ith ar model;
C        NOBS(I) = number of data falling in the Ith piece;
C Output: SIMV(IS+1)-SIMV(N) contains the normalized residuals.
C SUBROUTINES CALLED DESACF,HIST,LINMLD,NORPLT
C********************************************************************

      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION ANORM(*)
      DIMENSION STV(*),SIMV(1000),CMATX(2,*),
     1      P(1000),DENORM(*)
      DIMENSION NOBS(*)
      DIMENSION IP1T(*),IP2T(*)
      INTEGER INDEX(1000)
      EXTERNAL ZEROVEC,IZEROVEC
      CALL ZEROVEC(SIMV,1000,1000)
      CALL IZEROVEC(INDEX,1000,1000)
	BNO=0.0
      DO 878 I=1,2
  878 ANORM(I)=0.0
      DO 2 I=IES,N
C.....Sorting.
	 IP=1
	 INDEX(I)=IP
C.....Calculation of fitted residuals:
	 NPARM=1+IP1T(IP)+IP2T(IP)*(IP1T(IP)+1)
	 T=DBLE(CMATX(IP,1))
	 IF (NPARM.LT.2) GOTO 8
C.....Create vectors P:
            DO 10 I2=1,IP1T(IP)
c               write(3,*) 1+i2,i-i2
 10            P(1+I2)=STV(I-I2)
            DO 12 I2=1,IP1T(IP)+1
              IM=IS+(I2-1)*1
c              write(3,*) IP1T(IP),I-IM
 12           P(IP1T(IP)+I2+1)=STV(I-IM)
C.....Calculate fitted values:
	 DO 5 K=2,NPARM
c            write(3,*) DBLE(CMATX(IP,K)),DBLE(P(K)),T
    5       T=DBLE(CMATX(IP,K))*DBLE(P(K))+T
    8     SIMV(I)=DBLE(STV(I))-T
c         write(3,*) i,stv(i),t,simv(i)
C.....Calculation of sum of square of residuals piece by piece.
C.....ANORM(I) finally stores the residual sum of squares in the Ith piece.            
            ANORM(IP)=ANORM(IP)+(DBLE(SIMV(I))*DBLE(SIMV(I)))
    2       CONTINUE
	    IEND=NTHD+1
	    DO 17 I=1,IEND
	       FNOB=NOBS(I)
	       ANORM(I)=ANORM(I)/DBLE(FNOB)
   17          ANORM(I)=SQRT(ANORM(I))
C.....ANORM(I) stores root mean square of residuals in the Ith piece.
      DO 27 I=IES,N
	 IP=INDEX(I)
	 II=I-IES+1
C.... Store the root mean square used for normalization in the vector 
C.....DENORM. DENORM is used later on to denormalize resampled residuals.
c      SIMV(II)=DBLE(SIMV(I))/ANORM(IP)
      SIMV(II)=DBLE(SIMV(I))
      DENORM(II)=ANORM(IP)
   27 CONTINUE
      RETURN
      END

      SUBROUTINE PRELIM(B0,D0,C4,ILX,B1,IMX,IP1,IP2,IS)
C*****************************************************************
C       PORPOSE: PROVIDE LNRC PRELIMINAR ESTIMATES
C     INPUT: B1, VECTOR OF ESTIMATES
C     OUTPUT: D0, VECTOR OF ESTIMATES NONMULTIPLICATIVE FORM
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION B0(*),D0(*),C4(*)
      DIMENSION B1(*)
      EXTERNAL ZEROVEC,EQVEC
C
      I5=IP1+IP2*(IP1+1)+1
      CALL ZEROVEC(B0,LMAX,IP1+IP2)
      DO 117 I=1,IP1+1
 117     B0(I)=B1(I)
      DO 118 I=1,IP2
 118     B0(IP1+I+1)=B1((I-1)*(IP1+1)+IP1+2)
      CALL ZEROVEC(D0,LMAX,I5)
      IF(IP1.GT.0) THEN
	 DO 13 I=1,IP1+1
 13         D0(I)=B0(I)
      END IF
      IF (IP2.GT.0) THEN
	   CALL ZEROVEC(C4,LMAX,IP1+IP2+1)
	   C4(1)=1.0
	   DO 14 I=1,IP1+IP2
 14         C4(I+1)=B0(I+1)
	   DO 15 I1=1,IP2
	      DO 15 I2=1,IP1+1
 15         D0(IP1+(IP1+1)*(I1-1)+I2+1)=C4(I2)*C4(IP1+1+I1)
      END IF
      CALL EQVEC(D0,B0,IMX,I5)
      CALL ZEROVEC(D0,IMX,IP1+IS*IP2+1)
      IF (IP1.GT.0) THEN
	 DO 30 I=1,IP1+1
 30         D0(I)=B0(I)
      END IF
      IF (IP2.GT.0) THEN
	 DO 31 I1=1,IP2
	    DO 31 I2=1,IP1+1
 31            D0(IS*I1+I2)=B0(IP1+I2+(I1-1)*(IP1+1)+1)
      END IF
      RETURN
	END

      SUBROUTINE MULTEST(A2,D0,C2,A0,ILX,A,Q,B,B2,ELP,B9,B11,IMX,
     1                   IP1,IP2,IS,i9)

C*****************************************************************
C     PORPOSE: CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS 
C              PARTIAL DERIVATIVES MULTIPLICATIVE TESTING
C     INPUT: A2, VECTOR OF SCORES
C            D0, NON MULTIPLICATIVE VECTOR OF ESTIMATES
C            A, INFORMATION MATRIX
C     OUTPUT: A0, VECTOR OF ADJUSTING FACTORS
C             B, MATRIX OF RESTRICTED ESTIMATION
C*****************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A2(*),C2(*),D0(*),A0(*)
      DIMENSION Q(IMX,*),B(IMX,*),A(IMX,*),B2(IMX,*),ELP(IMX,*)
      DIMENSION B9(IMX,*),B11(IMX,*)
      EXTERNAL ZEROVEC,EQVEC,ZEROMAT,MULTMAT,MATINV,EQMAT
C
C       CONSTRUCTION OF THE VECTOR OF CONSTRAINTS AND ITS PARTIAL
C       DERIVATIVES
C
      I5=IP1+IP2*(IP1+1)+2
      ik=1.0
c     if (i9.eq.0) ik=-1.0
      CALL ZEROVEC(C2,IMX,I5+IP1*IP2)
         DO 50 I=1,I5
 50         C2(I)=A2(I)
      IF (IP1.GT.0.AND.IP2.GT.0) THEN
	   DO 52 I1=1,IP2
	      DO 52 I2=1,IP1
 52        C2(I5+IP1*(I1-1)+I2)=D0(IS*I1+I2+1)-D0(IS*I1+1)*D0(I2+1)*ik
      END IF
      write(3,*) (c2(ik),ik=1,i5+1)
      CALL ZEROMAT(Q,IMX,I5,IP1*IP2)
      IF (IP1.GT.0.AND.IP2.GT.0) THEN
         DO 55 I1=1,IP1*IP2
            Q(I5,I1)=0.0
 55         Q(1,I1)=0.0 
         DO 56 I1=1,IP2
            DO 56 I2=1,IP1
		 Q(I2+1,IP1*(I1-1)+I2)=D0(IS*I1+1)
		 Q(1+(IP1+1)*I1,IP1*(I1-1)+I2)=D0(I2+1)
 56        Q(1+(IP1+1)*I1+I2,IP1*(I1-1)+I2)=-1.0
      END IF
      CALL ZEROMAT(B,IMX,I5+IP1*IP2,I5+IP1*IP2)
      CALL ZEROMAT(B2,IMX,I5+IP1*IP2,I5+IP1*IP2)
      DO 60 I1=1,I5
         DO 60 I2=I1,I5
            B(I1,I2)=A(I1,I2)
 60         B(I2,I1)=A(I1,I2)
      DO 64 I1=1,I5
	   DO 64 I2=1,IP1*IP2
		B(I1,I5+I2)=Q(I1,I2)
 64         B(I2+I5,I1)=B(I1,I5+I2)
      CALL ZEROMAT(ELP,IMX,I5+IP1*IP2,I5+IP1*IP2)
      CALL MATINV(B,B2,ELP,IMX,I5+IP1*IP2,IFAIL)
      CALL EQMAT(B2,B,IMX,I5+IP1*IP2,I5+IP1*IP2)     
      CALL ZEROMAT(B9,IMX,IMX,1)
      CALL ZEROMAT(B11,IMX,IMX,1)
      DO 70 I=1,I5+IP1*IP2
 70      B9(I,1)=C2(I)
      CALL MULTMAT(B,B9,B11,IMX,IMX,I5+IP1*IP2,I5+IP1*IP2,1)
      CALL ZEROVEC(A0,IMX,I5+IP1*IP2)
      DO 72 I=1,I5+IP1*IP2
 72      A0(I)=B11(I,1)
      RETURN
      END

      SUBROUTINE MRQCOF(X,LX,SIG,NDATA,A,MA,ALPHA,BETA,
     1          NALP)
C**************************************************************
C     THIS SUBROUTINE PROVIDES THE SCORES AND THE INFORMATION
C     MATRIX OF EACH REGIME.
C     INPUT: X IS THE MATRIX OF DATA EQUAL TO THE SAX MATRIX (OUTPUT OF
C            THE 'SORT' SUBROUTINE.
C            A, VECTOR OF PRELIMINAR ESTIMATORS OF PARAMETERS    
C            SIG IS THE PRELIMINAR ESTIMATOR OF SIGMA SQUARE
C            MA # OF PARAMETERS ( CONSTANT + AR ORDER ) ( WE USE MA+1 TO
C            INCLUDE SIGMA-SQUARE DERIVATIVES )
C     OUTPUT: ALPHA INFORMATION MATRIX
C             BETA VECTOR OF SCORES
C     EXTERNAL FUNCTIONS:
C            FUNCS EVALUATES THE DERIVATIVES AND SECOND DERIVATIVES @ EACH
C             POINT
C**************************************************************

      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(LX,*),ALPHA(NALP,*),XI(100),D2YDA(200,200)
     1         ,A(*),BETA(*),DYDA(100)
      EXTERNAL FUNCS,ZEROVEC,ZEROMAT
      CALL ZEROVEC(BETA,100,100)
      CALL ZEROMAT(ALPHA,100,100,100)
      CALL ZEROMAT(D2YDA,200,100,100)
      CALL ZEROVEC(XI,100,100)
      DO 12 J=1,MA+1
	   DO 11 K=1,J
 11         ALPHA(J,K)=0.0
 12      BETA(J)=0.0
      PF3=0.0
      PF4=0.0
      DO 15 I=1,NDATA
	 XI(MA)=X(I,MA+1)         
	 DO 20 IQ=2,MA
 20         XI(MA+1-IQ)=X(I,IQ)
      CALL FUNCS(XI,A,YMOD,DYDA,MA,SIG,D2YDA,200,pf3)      
      DO 14 J=1,MA+1
	 DO 13 K=1,J
 13         ALPHA(J,K)=ALPHA(J,K)+D2YDA(J,K)
      BETA(J)=BETA(J)+DYDA(J)
 14   CONTINUE
      PF4=PF4+PF3
 15   CONTINUE 
      BETA(MA+1)=(-0.5*(NDATA*(SIG-(A(1)**2.0))-PF4))/(SIG**2.0)
      DO 19 L=1,MA+1
 19       BETA(L)=BETA(L)/DBLE(NDATA)
      DO 16 J=2,MA+1
	   DO 16 K=1,J-1
 16         ALPHA(K,J)=ALPHA(J,K)
      ALPHA(MA+1,MA+1)=(0.5*(NDATA*(SIG-(2*(A(1)**2.0))-
     1        PF4)))/(SIG**3.0)
      DO 18 J=1,MA+1
	 DO 18 K=1,MA+1
 18         ALPHA(J,K)=-ALPHA(J,K)/DBLE(NDATA)
      RETURN
      END

      SUBROUTINE FUNCS(X,A,Y,DYDA,NA,S2,D2YDA,NALP,pf3)
C**********************************************************************
C     THIS SUBROUTINE EVALUATES THE DERIVATIVES AND SECOND DERIVATIVES OF
C     THE LOG-LIKELIHOOD FUNCTION OF AN AR PROCESS 
C     INPUT: X, VECTOR WITH THE VALUE X(T) AND THE REGRESSORS NEEDED 
C            TO FORECAST X(T). ( X(NA)=X(T) X(NA-1)=X(T-1)...X(1)=X(T-P) )
C            A, VECTOR OF PRELIMINAR ESTIMATORS OF PARAMETERS
C            S2, SIGMA SQUARE VALUE
C     OUTPUT: DYDA, DERIVATIVE VECTOR @ X(T) 
C             D2YDA, SECOND DERIVATIVES MATRIX @ X(T)
C***********************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(*),DYDA(*),X(*),D2YDA(NALP,*)
      EXTERNAL ZEROVEC,ZEROMAT
      Y=0.0
      Q=0.0
      R=0.0
      PA=0.0
      PB=0.0
      PC=0.0
      PD=0.0
      PE=0.0
      PF=0.0
      PF1=0.0
      PF2=0.0
      PF3=0.0
      CALL ZEROVEC(DYDA,10,NA+1)
      CALL ZEROMAT(D2YDA,NALP,NA+1,NA+1)
      DO 1 J=2,NA
	 Q=Q+(X(NA-J+1)*A(J))
 1       R=R+((X(NA-J+1)**2.0)*(A(J)**2.0))
      PF=X(NA)-(A(1)+Q)
      pf1=x(na)-q
      pf2=pf1-2.0*a(1)
      pf3=pf1*pf2
C     *** derivative w.r.t constant ***
      DYDA(1)=PF/S2
C     *** derivatives w.r.t. AR parameters ***

      DO 2 K=2,NA
 2        DYDA(K)=(PF*X(NA-K+1))/S2

C     *** second derivative w.r.t constant ***

      D2YDA(1,1)=-1.0/S2

C     *** second derivatives w.r.t constant and parameters ***

      DO 3 L=2,NA
 3       D2YDA(L,1)=-X(NA-L+1)/S2

C     *** second derivative w.r.t constant and sigma square ***
 
      D2YDA(NA+1,1)=-PF/(S2**2.0)

C     *** second derivatives of parameters ***

      DO 4 L=2,NA
	 DO 4 K=2,L
 4          D2YDA(L,K)=-(X(NA-L+1)*X(NA-K+1))/S2

C     *** second derivative of sigma-square ***

c      D2YDA(NA+1,NA+1)=(-S2+4.0*PE)/(-2.0*(S2**3.0))

C     *** second derivative of sigma-square and parameters ***

      DO 6 L=2,NA
 6       D2YDA(NA+1,L)=-(PF*X(NA-L+1))/(S2**2.0)
      RETURN
      END

      SUBROUTINE MATINV(AMAT,BMAT,ELP,ID1MAX,L1,IFAIL)
C************************************************************************
C     PORPOSE: MATRIX INVERSION
C     INPUT: AMAT, MATRIX DIMENSION L1,L1
C     OUTPUT: BMAT, MATRIX DIMENSION L1,L1
C             IFAIL, MONITORING VALUE
C************************************************************************

      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AMAT(ID1MAX,*),BMAT(ID1MAX,*),ELP(ID1MAX,*)
      EXTERNAL EQMAT,ZEROMAT
C     
      CALL ZEROMAT(ELP,ID1MAX,L1,L1)
      CALL EQMAT(AMAT,ELP,ID1MAX,L1,L1)
      DO 115 I=1,L1
	 BMAT(I,I)=1.0
  115 CONTINUE
      DO 99 J=1,L1
	 DO 98 I=J,L1
	    IF (ELP(I,J).NE.0.0) GOTO 11
   98    CONTINUE
	 IFAIL=1
	 RETURN
   11    CONTINUE
      DO 97 K=1,L1
C
	 PIVOT01= ELP(J,K)
	 ELP(J,K)= ELP(I,K)
	 ELP(I,K)= PIVOT01
C
	 PIV= BMAT(J,K)
	 BMAT(J,K)=BMAT(I,K)
	 BMAT(I,K)= PIV
   97 CONTINUE
C
      IF(ELP(J,J).NE.0.0) GOTO 22
C
      IFAIL=1
      RETURN
C
C
   22 CONTINUE
      PPP= 1.0/ELP(J,J)
      DO 96 K=1,L1
         ELP(J,K)= ELP(J,K)*PPP
	 BMAT(J,K)= BMAT(J,K)*PPP
   96 CONTINUE
C
      DO 95 K=1,L1
C
	 IF(J.EQ.K) THEN
C
   33       CONTINUE
C
	 ELSE
	    PPP= ELP(K,J)
	    DO 94 L=1,L1
C
	       ELP(K,L)=ELP(K,L)-ELP(J,L)*PPP
	       BMAT(K,L)=BMAT(K,L)-BMAT(J,L)*PPP
   94       CONTINUE
C
	 END IF
   95 CONTINUE
   99 CONTINUE
C
      RETURN
      END


